/*
	Revolution-FX - Hardware Library for the fx-9860SD, fx-9860G, fx-9860AU, Graph85 and Graph85SD
	Written by kucalc
	
	Current Version: v0.3
	Contact: kucalc@gmail.com
	Homepage: http://revolution-fx.sourceforge.net
	Wiki: http://revolution-fx.sourceforge.net/wiki
	
	This work is licensed under the Creative Commons Attribution-Noncommercial-Share Alike 3.0 License. 
	To view a copy of this license, visit http://creativecommons.org/licenses/by-nc-sa/3.0/ or send a 
	letter to Creative Commons, 171 Second Street, Suite 300, San Francisco, California, 94105, USA.
*/

#include "fxlib.h"
#include "revolution.h"

#define INT_OFFSET 0x600

extern char info_text[2][27] = {
	"Created with Revolution-FX",
      "Version 0.3",
};

#define set_vbr(base)               _builtin_set_vbr(base)
#define sgn(x)				((x<0)?-1:((x>0)?1:0))

struct st_cpg {                                         /* struct CPG   */
              union {                                   /* FRQCR        */
                    unsigned short WORD;                /*  Word Access */
                    struct {                            /*  Bit  Access */
                           unsigned short      :3;      /*              */
                           unsigned short CKOEN:1;      /*    CKOEN     */
                           unsigned short      :2;      /*              */
                           unsigned short STC  :2;      /*    STC       */
                           unsigned short      :2;      /*              */
                           unsigned short IFC  :2;      /*    IFC       */
                           unsigned short      :2;      /*              */
                           unsigned short _PFC :2;      /*    PFC       */
                           }       BIT;                 /*              */
                    }           FRQCR;                  /*              */
};                                                      /*              */
struct st_wdt {                                         /* struct WDT   */
              union {                                   /* WTCNT        */
                    unsigned char  READ;                /*  Read  Access*/
                    unsigned short WRITE;               /*  Write Access*/
                    }           WTCNT;                  /*              */
              union {                                   /* WTCSR        */
                    union {                             /*  Read  Access*/
                          unsigned char BYTE;           /*   Byte Access*/
                          struct {                      /*   Bit  Access*/
                                 unsigned char TME :1;  /*    TME       */
                                 unsigned char WTIT:1;  /*    WT/IT     */
                                 unsigned char RSTS:1;  /*    RSTS      */
                                 unsigned char WOVF:1;  /*    WOVF      */
                                 unsigned char IOVF:1;  /*    IOVF      */
                                 unsigned char CKS :3;  /*    CKS       */
                                 }      BIT;            /*              */
                         }         READ;                /*              */
                    unsigned short WRITE;               /*  Write Access*/
                    }           WTCSR;                  /*              */
};

struct st_pa {                                          /* struct PA    */
             union {                                    /*              */
                   unsigned char BYTE;                  /*  Byte Access */
                   struct {                             /*  Bit  Access */
                          unsigned char B7:1;           /*    Bit 7     */
                          unsigned char B6:1;           /*    Bit 6     */
                          unsigned char B5:1;           /*    Bit 5     */
                          unsigned char B4:1;           /*    Bit 4     */
                          unsigned char B3:1;           /*    Bit 3     */
                          unsigned char B2:1;           /*    Bit 2     */
                          unsigned char B1:1;           /*    Bit 1     */
                          unsigned char B0:1;           /*    Bit 0     */
                          }      BIT;                   /*              */
                   }            DR;                     /*              */
};                                                      /*              */
struct st_pb {                                          /* struct PB    */
             union {                                    /*              */
                   unsigned char BYTE;                  /*  Byte Access */
                   struct {                             /*  Bit  Access */
                          unsigned char B7:1;           /*    Bit 7     */
                          unsigned char B6:1;           /*    Bit 6     */
                          unsigned char B5:1;           /*    Bit 5     */
                          unsigned char B4:1;           /*    Bit 4     */
                          unsigned char B3:1;           /*    Bit 3     */
                          unsigned char B2:1;           /*    Bit 2     */
                          unsigned char B1:1;           /*    Bit 1     */
                          unsigned char B0:1;           /*    Bit 0     */
                          }      BIT;                   /*              */
                   }            DR;                     /*              */
};                                                      /*              */
struct st_pc {                                          /* struct PC    */
             union {                                    /*              */
                   unsigned char BYTE;                  /*  Byte Access */
                   struct {                             /*  Bit  Access */
                          unsigned char B7:1;           /*    Bit 7     */
                          unsigned char B6:1;           /*    Bit 6     */
                          unsigned char B5:1;           /*    Bit 5     */
                          unsigned char B4:1;           /*    Bit 4     */
                          unsigned char B3:1;           /*    Bit 3     */
                          unsigned char B2:1;           /*    Bit 2     */
                          unsigned char B1:1;           /*    Bit 1     */
                          unsigned char B0:1;           /*    Bit 0     */
                          }      BIT;                   /*              */
                   }            DR;                     /*              */
};                                                      /*              */
struct st_pd {                                          /* struct PD    */
             union {                                    /*              */
                   unsigned char BYTE;                  /*  Byte Access */
                   struct {                             /*  Bit  Access */
                          unsigned char B7:1;           /*    Bit 7     */
                          unsigned char B6:1;           /*    Bit 6     */
                          unsigned char B5:1;           /*    Bit 5     */
                          unsigned char B4:1;           /*    Bit 4     */
                          unsigned char B3:1;           /*    Bit 3     */
                          unsigned char B2:1;           /*    Bit 2     */
                          unsigned char B1:1;           /*    Bit 1     */
                          unsigned char B0:1;           /*    Bit 0     */
                          }      BIT;                   /*              */
                   }            DR;                     /*              */
};                                                      /*              */
struct st_pe {                                          /* struct PE    */
             union {                                    /*              */
                   unsigned char BYTE;                  /*  Byte Access */
                   struct {                             /*  Bit  Access */
                          unsigned char B7:1;           /*    Bit 7     */
                          unsigned char B6:1;           /*    Bit 6     */
                          unsigned char B5:1;           /*    Bit 5     */
                          unsigned char B4:1;           /*    Bit 4     */
                          unsigned char B3:1;           /*    Bit 3     */
                          unsigned char B2:1;           /*    Bit 2     */
                          unsigned char B1:1;           /*    Bit 1     */
                          unsigned char B0:1;           /*    Bit 0     */
                          }      BIT;                   /*              */
                   }            DR;                     /*              */
};                                                      /*              */
struct st_pf {                                          /* struct PF    */
             union {                                    /*              */
                   unsigned char BYTE;                  /*  Byte Access */
                   struct {                             /*  Bit  Access */
                          unsigned char B7:1;           /*    Bit 7     */
                          unsigned char B6:1;           /*    Bit 6     */
                          unsigned char B5:1;           /*    Bit 5     */
                          unsigned char B4:1;           /*    Bit 4     */
                          unsigned char B3:1;           /*    Bit 3     */
                          unsigned char B2:1;           /*    Bit 2     */
                          unsigned char B1:1;           /*    Bit 1     */
                          unsigned char B0:1;           /*    Bit 0     */
                          }      BIT;                   /*              */
                   }            DR;                     /*              */
};                                                      /*              */
struct st_pg {                                          /* struct PG    */
             union {                                    /*              */
                   unsigned char BYTE;                  /*  Byte Access */
                   struct {                             /*  Bit  Access */
                          unsigned char B7:1;           /*    Bit 7     */
                          unsigned char B6:1;           /*    Bit 6     */
                          unsigned char B5:1;           /*    Bit 5     */
                          unsigned char B4:1;           /*    Bit 4     */
                          unsigned char B3:1;           /*    Bit 3     */
                          unsigned char B2:1;           /*    Bit 2     */
                          unsigned char B1:1;           /*    Bit 1     */
                          unsigned char B0:1;           /*    Bit 0     */
                          }      BIT;                   /*              */
                   }            DR;                     /*              */
};                                                      /*              */
struct st_ph {                                          /* struct PH    */
             union {                                    /*              */
                   unsigned char BYTE;                  /*  Byte Access */
                   struct {                             /*  Bit  Access */
                          unsigned char   :1;           /*    Bit 7     */
                          unsigned char B6:1;           /*    Bit 6     */
                          unsigned char B5:1;           /*    Bit 5     */
                          unsigned char B4:1;           /*    Bit 4     */
                          unsigned char B3:1;           /*    Bit 3     */
                          unsigned char B2:1;           /*    Bit 2     */
                          unsigned char B1:1;           /*    Bit 1     */
                          unsigned char B0:1;           /*    Bit 0     */
                          }      BIT;                   /*              */
                   }            DR;                     /*              */
};                                                      /*              */
struct st_pj {                                          /* struct PJ    */
             union {                                    /*              */
                   unsigned char BYTE;                  /*  Byte Access */
                   struct {                             /*  Bit  Access */
                          unsigned char B7:1;           /*    Bit 7     */
                          unsigned char B6:1;           /*    Bit 6     */
                          unsigned char B5:1;           /*    Bit 5     */
                          unsigned char B4:1;           /*    Bit 4     */
                          unsigned char B3:1;           /*    Bit 3     */
                          unsigned char B2:1;           /*    Bit 2     */
                          unsigned char B1:1;           /*    Bit 1     */
                          unsigned char B0:1;           /*    Bit 0     */
                          }      BIT;                   /*              */
                   }            DR;                     /*              */
};                                                      /*              */
struct st_pk {                                          /* struct PK    */
             union {                                    /*              */
                   unsigned char BYTE;                  /*  Byte Access */
                   struct {                             /*  Bit  Access */
                          unsigned char B7:1;           /*    Bit 7     */
                          unsigned char B6:1;           /*    Bit 6     */
                          unsigned char B5:1;           /*    Bit 5     */
                          unsigned char B4:1;           /*    Bit 4     */
                          unsigned char B3:1;           /*    Bit 3     */
                          unsigned char B2:1;           /*    Bit 2     */
                          unsigned char B1:1;           /*    Bit 1     */
                          unsigned char B0:1;           /*    Bit 0     */
                          }      BIT;                   /*              */
                   }            DR;                     /*              */
};                                                      /*              */
struct st_pl {                                          /* struct PL    */
             union {                                    /*              */
                   unsigned char BYTE;                  /*  Byte Access */
                   struct {                             /*  Bit  Access */
                          unsigned char   :4;           /*    Bit 7-4   */
                          unsigned char B3:1;           /*    Bit 3     */
                          unsigned char B2:1;           /*    Bit 2     */
                          unsigned char B1:1;           /*    Bit 1     */
                          unsigned char B0:1;           /*    Bit 0     */
                          }      BIT;                   /*              */
                   }            DR;                     /*              */
};                                                      /*              */
struct st_scp {                                         /* struct SCP   */
              union {                                   /*              */
                    unsigned char BYTE;                 /*  Byte Access */
                    struct {                            /*  Bit  Access */
                           unsigned char   :2;          /*    Bit 7,6   */
                           unsigned char B5:1;          /*    Bit 5     */
                           unsigned char B4:1;          /*    Bit 4     */
                           unsigned char B3:1;          /*    Bit 3     */
                           unsigned char B2:1;          /*    Bit 2     */
                           unsigned char B1:1;          /*    Bit 1     */
                           unsigned char B0:1;          /*    Bit 0     */
                           }      BIT;                  /*              */
                    }           DR;                     /*              */
};                                                      /*              */
struct st_pm {                                          /* struct PM    */
             union {                                    /*              */
                   unsigned char BYTE;                  /*  Byte Access */
                   struct {                             /*  Bit  Access */
                          unsigned char   :1;           /*    Bit 7     */
                          unsigned char B6:1;           /*    Bit 6     */
                          unsigned char   :1;           /*    Bit 5     */
                          unsigned char B4:1;           /*    Bit 4     */
                          unsigned char B3:1;           /*    Bit 3     */
                          unsigned char B2:1;           /*    Bit 2     */
                          unsigned char B1:1;           /*    Bit 1     */
                          unsigned char B0:1;           /*    Bit 0     */
                          }      BIT;                   /*              */
                   }            DR;                     /*              */
};                                                      /*              */
struct st_pn {                                          /* struct PN    */
             union {                                    /*              */
                   unsigned char BYTE;                  /*  Byte Access */
                   struct {                             /*  Bit  Access */
                          unsigned char B7:1;           /*    Bit 7     */
                          unsigned char B6:1;           /*    Bit 6     */
                          unsigned char B5:1;           /*    Bit 5     */
                          unsigned char B4:1;           /*    Bit 4     */
                          unsigned char B3:1;           /*    Bit 3     */
                          unsigned char B2:1;           /*    Bit 2     */
                          unsigned char B1:1;           /*    Bit 1     */
                          unsigned char B0:1;           /*    Bit 0     */
                          }      BIT;                   /*              */
                   }            DR;                     /*              */
};                                                      /*              */

struct st_rtc {                                         /* struct RTC   */
              unsigned char     R64CNT;                 /* R64CNT       */
              char              wk1;                    /*              */
              union {                                   /* RSECCNT      */
                    unsigned char BYTE;                 /*  Byte Access */
                    struct {                            /*  Bit  Access */
                           unsigned char    :1;         /*              */
                           unsigned char S10:3;         /*    10sec     */
                           unsigned char S1 :4;         /*     1sec     */
                           }      BIT;                  /*              */
                    }           RSECCNT;                /*              */
              char              wk2;                    /*              */
              union {                                   /* RMINCNT      */
                    unsigned char BYTE;                 /*  Byte Access */
                    struct {                            /*  Bit  Access */
                           unsigned char    :1;         /*              */
                           unsigned char M10:3;         /*    10min     */
                           unsigned char M1 :4;         /*     1min     */
                           }      BIT;                  /*              */
                    }           RMINCNT;                /*              */
              char              wk3;                    /*              */
              union {                                   /* RHRCNT       */
                    unsigned char BYTE;                 /*  Byte Access */
                    struct {                            /*  Bit  Access */
                           unsigned char    :2;         /*              */
                           unsigned char H10:2;         /*    10sec     */
                           unsigned char H1 :4;         /*     1sec     */
                           }      BIT;                  /*              */
                    }           RHRCNT;                 /*              */
              char              wk4;                    /*              */
              union {                                   /* RWKCNT       */
                    unsigned char BYTE;                 /*  Byte Access */
                    struct {                            /*  Bit  Access */
                           unsigned char   :5;          /*              */
                           unsigned char WK:3;          /*    week      */
                           }      BIT;                  /*              */
                    }           RWKCNT;                 /*              */
              char              wk5;                    /*              */
              union {                                   /* RDAYCNT      */
                    unsigned char BYTE;                 /*  Byte Access */
                    struct {                            /*  Bit  Access */
                           unsigned char    :2;         /*              */
                           unsigned char D10:2;         /*    10day     */
                           unsigned char D1 :4;         /*     1day     */
                           }      BIT;                  /*              */
                    }           RDAYCNT;                /*              */
              char              wk6;                    /*              */
              union {                                   /* RMONCNT      */
                    unsigned char BYTE;                 /*  Byte Access */
                    struct {                            /*  Bit  Access */
                           unsigned char    :3;         /*              */
                           unsigned char M10:1;         /*    10mon     */
                           unsigned char M1 :4;         /*     1mon     */
                           }      BIT;                  /*              */
                    }          RMONCNT;                 /*              */
              char             wk7;                     /*              */
              union {                                   /* RYRCNT       */
                    unsigned short WORD;                /*  Word Access */
                    struct {                            /*  Bit  Access */
                           unsigned short Y1000:4;      /*   1000year   */
                           unsigned short Y100 :4;      /*    100year   */
                           unsigned short Y10  :4;      /*     10year   */
                           unsigned short Y1   :4;      /*      1year   */
                           }       BIT;                 /*              */
                    }           RYRCNT;                 /*              */
              union {                                   /* RSECAR       */
                    unsigned char BYTE;                 /*  Byte Access */
                    struct {                            /*  Bit  Access */
                           unsigned char ENB:1;         /*    ENB       */
                           unsigned char S10:3;         /*    10sec     */
                           unsigned char S1 :4;         /*     1sec     */
                           }      BIT;                  /*              */
                    }           RSECAR;                 /*              */
              char              wk8;                    /*              */
              union {                                   /* RMINAR       */
                    unsigned char BYTE;                 /*  Byte Access */
                    struct {                            /*  Bit  Access */
                           unsigned char ENB:1;         /*    ENB       */
                           unsigned char M10:3;         /*    10min     */
                           unsigned char M1 :4;         /*     1min     */
                           }      BIT;                  /*              */
                    }           RMINAR;                 /*              */
              char              wk9;                    /*              */
              union {                                   /* RHRAR        */
                    unsigned char BYTE;                 /*  Byte Access */
                    struct {                            /*  Bit  Access */
                           unsigned char ENB:1;         /*    ENB       */
                           unsigned char    :1;         /*              */
                           unsigned char H10:2;         /*    10sec     */
                           unsigned char H1 :4;         /*     1sec     */
                           }      BIT;                  /*              */
                    }           RHRAR;                  /*              */
              char              wk10;                   /*              */
              union {                                   /* RWKAR        */
                    unsigned char BYTE;                 /*  Byte Access */
                    struct {                            /*  Bit  Access */
                           unsigned char ENB:1;         /*    ENB       */
                           unsigned char    :4;         /*              */
                           unsigned char WK :3;         /*    week      */
                           }      BIT;                  /*              */
                    }           RWKAR;                  /*              */
              char              wk11;                   /*              */
              union {                                   /* RDAYAR       */
                    unsigned char BYTE;                 /*  Byte Access */
                    struct {                            /*  Bit  Access */
                           unsigned char ENB:1;         /*    ENB       */
                           unsigned char    :1;         /*              */
                           unsigned char D10:2;         /*    10day     */
                           unsigned char D1 :4;         /*     1day     */
                           }      BIT;                  /*              */
                    }           RDAYAR;                 /*              */
              char              wk12;                   /*              */
              union {                                   /* RMONAR       */
                    unsigned char BYTE;                 /*  Byte Access */
                    struct {                            /*  Bit  Access */
                           unsigned char ENB:1;         /*    ENB       */
                           unsigned char    :2;         /*              */
                           unsigned char M10:1;         /*    10mon     */
                           unsigned char M1 :4;         /*     1mon     */
                           }      BIT;                  /*              */
                    }           RMONAR;                 /*              */
              char              wk13;                   /*              */
              union {                                   /* RCR1         */
                    unsigned char BYTE;                 /*  Byte Access */
                    struct {                            /*  Bit  Access */
                           unsigned char CF :1;         /*    CF        */
                           unsigned char    :2;         /*              */
                           unsigned char CIE:1;         /*    CIE       */
                           unsigned char AIE:1;         /*    AIE       */
                           unsigned char    :2;         /*              */
                           unsigned char AF :1;         /*    AF        */
                           }      BIT;                  /*              */
                    }           RCR1;                   /*              */
              char              wk14;                   /*              */
              union {                                   /* RCR2         */
                    unsigned char BYTE;                 /*  Byte Access */
                    struct {                            /*  Bit  Access */
                           unsigned char PEF  :1;       /*    PEF       */
                           unsigned char PES  :3;       /*    PES       */
                           unsigned char RTCEN:1;       /*    RTCEN     */
                           unsigned char ADJ  :1;       /*    ADJ       */
                           unsigned char RESET:1;       /*    RESET     */
                           unsigned char START:1;       /*    START     */
                           }      BIT;                  /*              */
                    }           RCR2;                   /*              */
};

struct st_tmu {                                         /* struct TMU   */
              union {                                   /* TSTR         */
                    unsigned char BYTE;                 /*  Byte Access */
                    struct {                            /*  Bit  Access */
                           unsigned char     :5;        /*              */
                           unsigned char STR2:1;        /*    STR2      */
                           unsigned char STR1:1;        /*    STR1      */
                           unsigned char STR0:1;        /*    STR0      */
                           }      BIT;                  /*              */
                    }           TSTR;                   /*              */
};                                                      /*              */
struct st_tmu0 {                                        /* struct TMU0  */
               unsigned int     TCOR;                   /* TCOR         */
               unsigned int     TCNT;                   /* TCNT         */
               union {                                  /* TCR          */
                     unsigned short WORD;               /*  Word Access */
                     struct {                           /*  Bit  Access */
                            unsigned short     :7;      /*              */
                            unsigned short UNF :1;      /*    UNF       */
                            unsigned short     :2;      /*              */
                            unsigned short UNIE:1;      /*    UNIE      */
                            unsigned short CKEG:2;      /*    CKEG      */
                            unsigned short TPSC:3;      /*    TPSC      */
                            }       BIT;                /*              */
                     }          TCR;                    /*              */
};                                                      /*              */
struct st_tmu2 {                                        /* struct TMU2  */
               unsigned int     TCOR;                   /* TCOR         */
               unsigned int     TCNT;                   /* TCNT         */
               union {                                  /* TCR          */
                     unsigned short WORD;               /*  Word Access */
                     struct {
                            unsigned short     :6;
                            unsigned short ICPF:1; 
                            unsigned short UNF :1;
                            unsigned short ICPE:2;
                            unsigned short UNIE:1;
                            unsigned short CKEG:2;
                            unsigned short TPSC:3;
                            }       BIT;
                     }          TCR;
               char             wk[2];
               unsigned int     TCPR;
};    

struct st_usb {                                         /* struct USB   */
              union {                                   /* UCLKCR       */
                    unsigned char BYTE;                 /*  Byte Access */
                    struct {                            /*  Bit  Access */
                           unsigned char USSCS:2;       /*    USSCS     */
                           unsigned char USBEN:1;       /*    USBEN     */
                           }      BIT;                  /*              */
                    }           UCLKCR;                 /*              */
              char              wk1[4063223];           /*              */
              unsigned char     EPDR0i;                 /* EPDR0i       */
              char              wk2[3];                 /*              */
              unsigned char     EPDR0o;                 /* EPDR0o       */
              char              wk3[3];                 /*              */
              unsigned char     EPDR0s;                 /* EPDR0s       */
              char              wk4[3];                 /*              */
              unsigned char     EPDR1;                  /* EPDR1        */
              char              wk5[3];                 /*              */
              unsigned char     EPDR2;                  /* EPDR2        */
              char              wk6[3];                 /*              */
              unsigned char     EPDR3;                  /* EPDR3        */
              char              wk7[3];                 /*              */
              union {                                   /* IFR0         */
                    unsigned char BYTE;                 /*  Byte Access */
                    struct {                            /*  Bit  Access */
                           unsigned char BRST    :1;    /*    BRST      */
                           unsigned char EP1FULL :1;    /*    EP1FULL   */
                           unsigned char EP2TR   :1;    /*    EP2TR     */
                           unsigned char EP2EMPTY:1;    /*    EP2EMPTY  */
                           unsigned char SETUPTS :1;    /*    SETUPTS   */
                           unsigned char EP0oTS  :1;    /*    EP0oTS    */
                           unsigned char EP0iTR  :1;    /*    EP0iTR    */
                           unsigned char EP0iTS  :1;    /*    EP0iTS    */
                           }      BIT;                  /*              */
                    }           IFR0;                   /*              */
              char              wk8[3];                 /*              */
              union {                                   /* IFR1         */
                    unsigned char BYTE;                 /*  Byte Access */
                    struct {                            /*  Bit  Access */
                           unsigned char       :4;      /*              */
                           unsigned char VBUSMN:1;      /*    VBUSMN    */
                           unsigned char EP3TS :1;      /*    EP3TS     */
                           unsigned char EP3TR :1;      /*    EP3TR     */
                           unsigned char VBUS  :1;      /*    VBUS      */
                           }      BIT;                  /*              */
                    }           IFR1;                   /*              */
              char              wk9[3];                 /*              */
              union {                                   /* TRG          */
                    unsigned char BYTE;                 /*  Byte Access */
                    struct {                            /*  Bit  Access */
                           unsigned char         :1;    /*              */
                           unsigned char EP3PKTE :1;    /*    EP3PKTE   */
                           unsigned char EP1RDFN :1;    /*    EP1RDFN   */
                           unsigned char EP2PKTE :1;    /*    EP2PKTE   */
                           unsigned char         :1;    /*              */
                           unsigned char EP0sRDFN:1;    /*    EP0sRDFN  */
                           unsigned char EP0oRDFN:1;    /*    EP0oRDFN  */
                           unsigned char EP0iPKTE:1;    /*    EP0iPKTE  */
                           }      BIT;                  /*              */
                    }           TRG;                    /*              */
              char              wk10[3];                /*              */
              union {                                   /* FCLR         */
                    unsigned char BYTE;                 /*  Byte Access */
                    struct {                            /*  Bit  Access */
                           unsigned char        :1;     /*              */
                           unsigned char EP3CLR :1;     /*    EP3CLR    */
                           unsigned char EP1CLR :1;     /*    EP1CLR    */
                           unsigned char EP2CLR :1;     /*    EP2CLR    */
                           unsigned char        :2;     /*              */
                           unsigned char EP0oCLR:1;     /*    EP0oCLR   */
                           unsigned char EP0iCLR:1;     /*    EP0iCLR   */
                           }      BIT;                  /*              */
                    }           FCLR;                   /*              */
              char              wk11[3];                /*              */
              unsigned char     EPSZ0o;                 /* EPSZ0o       */
              char              wk12[3];                /*              */
              union {                                   /* DASTS        */
                    unsigned char BYTE;                 /*  Byte Access */
                    struct {                            /*  Bit  Access */
                           unsigned char       :2;      /*              */
                           unsigned char EP3DE :1;      /*    EP3DE     */
                           unsigned char EP2DE :1;      /*    EP2DE     */
                           unsigned char       :3;      /*              */
                           unsigned char EP0iDE:1;      /*    EP0iDE    */
                           }      BIT;                  /*              */
                    }           DASTS;                  /*              */
              char              wk13[3];                /*              */
              union {                                   /* EPSTL        */
                    unsigned char BYTE;                 /*  Byte Access */
                    struct {                            /*  Bit  Access */
                           unsigned char       :4;      /*              */
                           unsigned char EP3STL:1;      /*    EP3STL    */
                           unsigned char EP2STL:1;      /*    EP2STL    */
                           unsigned char EP1STL:1;      /*    EP1STL    */
                           unsigned char EP0STL:1;      /*    EP0STL    */
                           }      BIT;                  /*              */
                    }           EPSTL;                  /*              */
              char              wk14[3];                /*              */
              union {                                   /* IER0         */
                    unsigned char BYTE;                 /*  Byte Access */
                    struct {                            /*  Bit  Access */
                           unsigned char BRST    :1;    /*    BRST      */
                           unsigned char EP1FULL :1;    /*    EP1FULL   */
                           unsigned char EP2TR   :1;    /*    EP2TR     */
                           unsigned char EP2EMPTY:1;    /*    EP2EMPTY  */
                           unsigned char SETUPTS :1;    /*    SETUPTS   */
                           unsigned char EP0oTS  :1;    /*    EP0oTS    */
                           unsigned char EP0iTR  :1;    /*    EP0iTR    */
                           unsigned char EP0iTS  :1;    /*    EP0iTS    */
                           }      BIT;                  /*              */
                    }           IER0;                   /*              */
              char              wk15[3];                /*              */
              union {                                   /* IER1         */
                    unsigned char BYTE;                 /*  Byte Access */
                    struct {                            /*  Bit  Access */
                           unsigned char      :5;       /*              */
                           unsigned char EP3TS:1;       /*    EP3TS     */
                           unsigned char EP3TR:1;       /*    EP3TR     */
                           unsigned char VBUS :1;       /*    VBUS      */
                           }      BIT;                  /*              */
                    }           IER1;                   /*              */
              char              wk16[3];                /*              */
              unsigned char     EPSZ1;                  /* EPSZ1        */
              char              wk17[3];                /*              */
              union {                                   /* DMAR         */
                    unsigned char BYTE;                 /*  Byte Access */
                    struct {                            /*  Bit  Access */
                           unsigned char        :6;     /*              */
                           unsigned char EP2DMAE:1;     /*    EP2DMAE   */
                           unsigned char EP1DMAE:1;     /*    EP1DMAE   */
                           }      BIT;                  /*              */
                    }           DMAR;                   /*              */
              char              wk18[3];                /*              */
              union {                                   /* ISR0         */
                    unsigned char BYTE;                 /*  Byte Access */
                    struct {                            /*  Bit  Access */
                           unsigned char BRST    :1;    /*    BRST      */
                           unsigned char EP1FULL :1;    /*    EP1FULL   */
                           unsigned char EP2TR   :1;    /*    EP2TR     */
                           unsigned char EP2EMPTY:1;    /*    EP2EMPTY  */
                           unsigned char SETUPTS :1;    /*    SETUPTS   */
                           unsigned char EP0oTS  :1;    /*    EP0oTS    */
                           unsigned char EP0iTR  :1;    /*    EP0iTR    */
                           unsigned char EP0iTS  :1;    /*    EP0iTS    */
                           }      BIT;                  /*              */
                    }           ISR0;                   /*              */
              char              wk19[3];                /*              */
              union {                                   /* ISR1         */
                    unsigned char BYTE;                 /*  Byte Access */
                    struct {                            /*  Bit  Access */
                           unsigned char      :5;       /*              */
                           unsigned char EP3TS:1;       /*    EP3TS     */
                           unsigned char EP3TR:1;       /*    EP3TR     */
                           unsigned char VBUS :1;       /*    VBUS      */
                           }      BIT;                  /*              */
                    }           ISR1;                   /*              */
              char              wk20[23];               /*              */
              union {                                   /* XVERCR       */
                    unsigned char BYTE;                 /*  Byte Access */
                    struct {                            /*  Bit  Access */
                           unsigned char        :6;     /*              */
                           unsigned char XVEROFF:1;     /*    XVEROFF   */
                           }      BIT;                  /*              */
                    }           XVERCR;                 /*              */
};
struct st_intc {                                        /* struct INTC  */
               union {                                  /* ICR0         */
                     unsigned short WORD;               /*  Word Access */
                     struct {                           /*  Bit  Access */
                            unsigned short NMIL:1;      /*    NMIL      */
                            unsigned short     :6;      /*              */
                            unsigned short NMIE:1;      /*    NMIE      */
                            }       BIT;                /*              */
                     }          ICR0;                   /*              */
               union {                                  /* IPRA         */
                     unsigned short WORD;               /*  Word Access */
                     struct {                           /*  Bit  Access */
                            unsigned short _TMU0:4;     /*    TMU0      */
                            unsigned short _TMU1:4;     /*    TMU1      */
                            unsigned short _TMU2:4;     /*    TMU2      */
                            unsigned short _RTC :4;     /*    RTC       */
                            }       BIT;                /*              */
                     }          IPRA;                   /*              */
               union {                                  /* IPRB         */
                     unsigned short WORD;               /*  Word Access */
                     struct {                           /*  Bit  Access */
                            unsigned short _WDT:4;      /*    WDT       */
                            unsigned short _REF:4;      /*    REF       */
                            }       BIT;                /*              */
                     }          IPRB;                   /*              */
               char             wk[234];                /*              */
               unsigned int     TRA;                    /* TRA          */
               unsigned int     EXPEVT;                 /* EXPEVT       */
               unsigned int     INTEVT;                 /* INTEVT       */
};
struct st_intx {                                        /* struct INTX  */
               unsigned int     INTEVT2;                /* INTEVT2      */
               union {                                  /* IRR0         */
                     unsigned char BYTE;                /*  Byte Access */
                     struct {                           /*  Bit  Access */
                            unsigned char PINT0R:1;     /*    PINT0R    */
                            unsigned char PINT1R:1;     /*    PINT1R    */
                            unsigned char IRQ5R :1;     /*    IRQ5R     */
                            unsigned char IRQ4R :1;     /*    IRQ4R     */
                            unsigned char IRQ3R :1;     /*    IRQ3R     */
                            unsigned char IRQ2R :1;     /*    IRQ2R     */
                            unsigned char IRQ1R :1;     /*    IRQ1R     */
                            unsigned char IRQ0R :1;     /*    IRQ0R     */
                            }      BIT;                 /*              */
                     }          IRR0;                   /*              */
               char             wk1;                    /*              */
               union {                                  /* IRR1         */
                     unsigned char BYTE;                /*  Byte Access */
                     struct {                           /*  Bit  Access */
                            unsigned char TXI0R:1;      /*    TXI0R     */
                            unsigned char      :1;      /*              */
                            unsigned char RXI0R:1;      /*    RXI0R     */
                            unsigned char ERI0R:1;      /*    ERI0R     */
                            unsigned char DEI3R:1;      /*    DEI3R     */
                            unsigned char DEI2R:1;      /*    DEI2R     */
                            unsigned char DEI1R:1;      /*    DEI1R     */
                            unsigned char DEI0R:1;      /*    DEI0R     */
                            }      BIT;                 /*              */
                     }          IRR1;                   /*              */
               char             wk2;                    /*              */
               union {                                  /* IRR2         */
                     unsigned char BYTE;                /*  Byte Access */
                     struct {                           /*  Bit  Access */
                            unsigned char      :3;      /*              */
                            unsigned char ADIR :1;      /*    ADIR      */
                            unsigned char TXI2R:1;      /*    TXI2R     */
                            unsigned char      :1;      /*              */
                            unsigned char RXI2R:1;      /*    RXI2R     */
                            unsigned char ERI2R:1;      /*    ERI2R     */
                            }      BIT;                 /*              */
                     }          IRR2;                   /*              */
               char             wk3[7];                 /*              */
               union {                                  /* ICR1         */
                     unsigned short WORD;               /*  Word Access */
                     struct {                           /*  Bit  Access */
                            unsigned short MAI   :1;    /*    MAI       */
                            unsigned short IRQLVL:1;    /*    IRQLVL    */
                            unsigned short BLMSK :1;    /*    BLMSK     */
                            unsigned short       :1;    /*              */
                            unsigned short IRQ5S :2;    /*    IRQ5S     */
                            unsigned short IRQ4S :2;    /*    IRQ4S     */
                            unsigned short IRQ3S :2;    /*    IRQ3S     */
                            unsigned short IRQ2S :2;    /*    IRQ2S     */
                            unsigned short IRQ1S :2;    /*    IRQ1S     */
                            unsigned short IRQ0S :2;    /*    IRQ0S     */
                            }       BIT;                /*              */
                     }          ICR1;                   /*              */
               union {                                  /* ICR2         */
                     unsigned short WORD;               /*  Word Access */
                     struct {                           /*  Bit  Access */
                            unsigned short PINT15S:1;   /*    PINT15S   */
                            unsigned short PINT14S:1;   /*    PINT14S   */
                            unsigned short PINT13S:1;   /*    PINT13S   */
                            unsigned short PINT12S:1;   /*    PINT12S   */
                            unsigned short PINT11S:1;   /*    PINT11S   */
                            unsigned short PINT10S:1;   /*    PINT10S   */
                            unsigned short PINT9S :1;   /*    PINT9S    */
                            unsigned short PINT8S :1;   /*    PINT8S    */
                            unsigned short PINT7S :1;   /*    PINT7S    */
                            unsigned short PINT6S :1;   /*    PINT6S    */
                            unsigned short PINT5S :1;   /*    PINT5S    */
                            unsigned short PINT4S :1;   /*    PINT4S    */
                            unsigned short PINT3S :1;   /*    PINT3S    */
                            unsigned short PINT2S :1;   /*    PINT2S    */
                            unsigned short PINT1S :1;   /*    PINT1S    */
                            unsigned short PINT0S :1;   /*    PINT0S    */
                            }       BIT;                /*              */
                     }          ICR2;                   /*              */
               union {                                  /* PINTER       */
                     unsigned short WORD;               /*  Word Access */
                     struct {                           /*  Bit  Access */
                            unsigned short PINT15E:1;   /*    PINT15E   */
                            unsigned short PINT14E:1;   /*    PINT14E   */
                            unsigned short PINT13E:1;   /*    PINT13E   */
                            unsigned short PINT12E:1;   /*    PINT12E   */
                            unsigned short PINT11E:1;   /*    PINT11E   */
                            unsigned short PINT10E:1;   /*    PINT10E   */
                            unsigned short PINT9E :1;   /*    PINT9E    */
                            unsigned short PINT8E :1;   /*    PINT8E    */
                            unsigned short PINT7E :1;   /*    PINT7E    */
                            unsigned short PINT6E :1;   /*    PINT6E    */
                            unsigned short PINT5E :1;   /*    PINT5E    */
                            unsigned short PINT4E :1;   /*    PINT4E    */
                            unsigned short PINT3E :1;   /*    PINT3E    */
                            unsigned short PINT2E :1;   /*    PINT2E    */
                            unsigned short PINT1E :1;   /*    PINT1E    */
                            unsigned short PINT0E :1;   /*    PINT0E    */
                            }       BIT;                /*              */
                     }          PINTER;                 /*              */
               union {                                  /* IPRC         */
                     unsigned short WORD;               /*  Word Access */
                     struct {                           /*  Bit  Access */
                            unsigned short _IRQ3:4;     /*    IRQ3      */
                            unsigned short _IRQ2:4;     /*    IRQ2      */
                            unsigned short _IRQ1:4;     /*    IRQ1      */
                            unsigned short _IRQ0:4;     /*    IRQ0      */
                            }       BIT;                /*              */
                     }          IPRC;                   /*              */
               union {                                  /* IPRD         */
                     unsigned short WORD;               /*  Word Access */
                     struct {                           /*  Bit  Access */
                            unsigned short _PINT0_7 :4; /*    PINT0-7   */
                            unsigned short _PINT8_15:4; /*    PINT8-15  */
                            unsigned short _IRQ5    :4; /*    IRQ5      */
                            unsigned short _IRQ4    :4; /*    IRQ4      */
                            }       BIT;                /*              */
                     }          IPRD;                   /*              */
               union {                                  /* IPRE         */
                     unsigned short WORD;               /*  Word Access */
                     struct {                           /*  Bit  Access */
                            unsigned short _DMAC :4;    /*    DMAC      */
                            unsigned short _SCIF0:4;    /*    SCIF0     */
                            unsigned short _SCIF2:4;    /*    SCIF2     */
                            unsigned short _ADC  :4;    /*    ADC       */
                            }       BIT;                /*              */
                     }          IPRE;                   /*              */
               char             wk4[524260];            /*              */
               union {                                  /* IPRF         */
                     unsigned short WORD;               /*  Word Access */
                     struct {                           /*  Bit  Access */
                            unsigned short     :8;      /*              */
                            unsigned short _USB:4;      /*    USB       */
                            }       BIT;                /*              */
                     }          IPRF;                   /*              */
               union {                                  /* IPRG         */
                     unsigned short WORD;               /*  Word Access */
                     struct {                           /*  Bit  Access */
                            unsigned short _TPU0:4;     /*    TPU0      */
                            unsigned short _TPU1:4;     /*    TPU1      */
                            }       BIT;                /*              */
                     }          IPRG;                   /*              */
               union {                                  /* IPRH         */
                     unsigned short WORD;               /*  Word Access */
                     struct {                           /*  Bit  Access */
                            unsigned short _TPU2:4;     /*    TPU2      */
                            unsigned short _TPU3:4;     /*    TPU3      */
                            }       BIT;                /*              */
                     }          IPRH;                   /*              */
}; 
struct st_scif {                                        /* struct SCIF  */
               union {                                  /* SCSMR        */
                     unsigned short WORD;               /*  Word Access */
                     struct {                           /*  Bit  Access */
                            unsigned short     :5;      /*              */
                            unsigned short SRC :3;      /*    SRC       */
                            unsigned short CA  :1;      /*    CA        */
                            unsigned short CHR :1;      /*    CHR       */
                            unsigned short _PE :1;      /*    PE        */
                            unsigned short OE  :1;      /*    O/E       */
                            unsigned short STOP:1;      /*    STOP      */
                            unsigned short     :1;      /*              */
                            unsigned short CKS :2;      /*    CKS       */
                            }       BIT;                /*              */
                     }          SCSMR;                  /*              */
               char             wk1[2];                 /*              */
               unsigned char    SCBRR;                  /* SCBRR        */
               char             wk2[3];                 /*              */
               union {                                  /* SCSCR        */
                     unsigned short WORD;               /*  Word Access */
                     struct {                           /*  Bit  Access */
                            unsigned short     :4;      /*              */
                            unsigned short TSIE:1;      /*    TSIE      */
                            unsigned short ERIE:1;      /*    ERIE      */
                            unsigned short BRIE:1;      /*    BRIE      */
                            unsigned short DRIE:1;      /*    DRIE      */
                            unsigned short TIE :1;      /*    TIE       */
                            unsigned short RIE :1;      /*    RIE       */
                            unsigned short TE  :1;      /*    TE        */
                            unsigned short RE  :1;      /*    RE        */
                            unsigned short     :2;      /*              */
                            unsigned short CKE :2;      /*    CKE       */
                            }       BIT;                /*              */
                     }          SCSCR;                  /*              */
               char             wk3[2];                 /*              */
               unsigned char    SCTDSR;                 /* SCTDSR       */
               char             wk4[3];                 /*              */
               union {                                  /* SCFER        */
                     unsigned short WORD;               /*  Word Access */
                     struct {                           /*  Bit  Access */
                            unsigned short    :2;       /*              */
                            unsigned short PER:6;       /*    PER       */
                            unsigned short    :2;       /*              */
                            unsigned short FER:6;       /*    FER       */
                            }       BIT;                /*              */
                     }          SCFER;                  /*              */
               char             wk5[2];                 /*              */
               union {                                  /* SCSSR        */
                     unsigned short WORD;               /*  Word Access */
                     struct {                           /*  Bit  Access */
                            unsigned short     :6;      /*              */
                            unsigned short ORER:1;      /*    ORER      */
                            unsigned short TSF :1;      /*    TSF       */
                            unsigned short ER  :1;      /*    ER        */
                            unsigned short TEND:1;      /*    TEND      */
                            unsigned short TDFE:1;      /*    TDFE      */
                            unsigned short BRK :1;      /*    BRK       */
                            unsigned short FER :1;      /*    FER       */
                            unsigned short PER :1;      /*    PER       */
                            unsigned short RDF :1;      /*    RDF       */
                            unsigned short DR  :1;      /*    DR        */
                            }       BIT;                /*              */
                     }          SCSSR;                  /*              */
               char             wk6[2];                 /*              */
               union {                                  /* SCFCR        */
                     unsigned short WORD;               /*  Word Access */
                     struct {                           /*  Bit  Access */
                            unsigned short TSE  :1;     /*    TSE       */
                            unsigned short TCRST:1;     /*    TCRST     */
                            unsigned short      :3;     /*              */
                            unsigned short RSTRG:3;     /*    RSTRG     */
                            unsigned short RTRG :2;     /*    RTRG      */
                            unsigned short TTRG :2;     /*    TTRG      */
                            unsigned short MCE  :1;     /*    MCE       */
                            unsigned short TFRST:1;     /*    TFRST     */
                            unsigned short RFRST:1;     /*    RFRST     */
                            unsigned short LOOP :1;     /*    LOOP      */
                            }       BIT;                /*              */
                     }          SCFCR;                  /*              */
               char             wk7[2];                 /*              */
               union {                                  /* SCFDR        */
                     unsigned short WORD;               /*  Word Access */
                     struct {                           /*  Byte Access */
                            unsigned short  :1;         /*              */
                            unsigned short T:7;         /*    T         */
                            unsigned short  :1;         /*              */
                            unsigned short R:7;         /*    R         */
                            }       BIT;                /*              */
                     }          SCFDR;                  /*              */
               char             wk8[2];                 /*              */
               unsigned char    SCFTDR;                 /* SCFTDR       */
               char             wk9[3];                 /*              */
               unsigned char    SCFRDR;                 /* SCFRDR       */
};                                                      /*              */
union un_stbcr3 {                                       /* union STBCR3 */
                unsigned char BYTE;                     /*  Byte Access */
                struct {                                /*  Bit  Access */
                       unsigned char _USB  :1;          /*    USB       */
                       unsigned char       :1;          /*              */
                       unsigned char _CMT  :1;          /*    CMT       */
                       unsigned char _TPU  :1;          /*    TPU       */
                       unsigned char _ADC  :1;          /*    ADC       */
                       unsigned char _IrDA :1;          /*    IrDA      */
                       unsigned char _SCIF2:1;          /*    SCIF2     */
                       unsigned char _SCIF0:1;          /*    SCIF0     */
                       }      BIT;                      /*              */
};                                                      /*              */
struct st_pfc {                                         /* struct PFC   */
              union {                                   /* PACR         */
                    unsigned short WORD;                /*  Word Access */
                    struct {                            /*  Bit  Access */
                           unsigned short PA7MD:2;      /*    PA7MD     */
                           unsigned short PA6MD:2;      /*    PA6MD     */
                           unsigned short PA5MD:2;      /*    PA5MD     */
                           unsigned short PA4MD:2;      /*    PA4MD     */
                           unsigned short PA3MD:2;      /*    PA3MD     */
                           unsigned short PA2MD:2;      /*    PA2MD     */
                           unsigned short PA1MD:2;      /*    PA1MD     */
                           unsigned short PA0MD:2;      /*    PA0MD     */
                           }       BIT;                 /*              */
                    }           PACR;                   /*              */
              union {                                   /* PBCR         */
                    unsigned short WORD;                /*  Word Access */
                    struct {                            /*  Bit  Access */
                           unsigned short PB7MD:2;      /*    PB7MD     */
                           unsigned short PB6MD:2;      /*    PB6MD     */
                           unsigned short PB5MD:2;      /*    PB5MD     */
                           unsigned short PB4MD:2;      /*    PB4MD     */
                           unsigned short PB3MD:2;      /*    PB3MD     */
                           unsigned short PB2MD:2;      /*    PB2MD     */
                           unsigned short PB1MD:2;      /*    PB1MD     */
                           unsigned short PB0MD:2;      /*    PB0MD     */
                           }       BIT;                 /*              */
                    }           PBCR;                   /*              */
              union {                                   /* PCCR         */
                    unsigned short WORD;                /*  Word Access */
                    struct {                            /*  Bit  Access */
                           unsigned short PC7MD:2;      /*    PC7MD     */
                           unsigned short PC6MD:2;      /*    PC6MD     */
                           unsigned short PC5MD:2;      /*    PC5MD     */
                           unsigned short PC4MD:2;      /*    PC4MD     */
                           unsigned short PC3MD:2;      /*    PC3MD     */
                           unsigned short PC2MD:2;      /*    PC2MD     */
                           unsigned short PC1MD:2;      /*    PC1MD     */
                           unsigned short PC0MD:2;      /*    PC0MD     */
                           }       BIT;                 /*              */
                    }           PCCR;                   /*              */
              union {                                   /* PDCR         */
                    unsigned short WORD;                /*  Word Access */
                    struct {                            /*  Bit  Access */
                           unsigned short PD7MD:2;      /*    PD7MD     */
                           unsigned short PD6MD:2;      /*    PD6MD     */
                           unsigned short PD5MD:2;      /*    PD5MD     */
                           unsigned short PD4MD:2;      /*    PD4MD     */
                           unsigned short PD3MD:2;      /*    PD3MD     */
                           unsigned short PD2MD:2;      /*    PD2MD     */
                           unsigned short PD1MD:2;      /*    PD1MD     */
                           unsigned short PD0MD:2;      /*    PD0MD     */
                           }       BIT;                 /*              */
                    }           PDCR;                   /*              */
              union {                                   /* PECR         */
                    unsigned short WORD;                /*  Word Access */
                    struct {                            /*  Bit  Access */
                           unsigned short PE7MD:2;      /*    PE7MD     */
                           unsigned short PE6MD:2;      /*    PE6MD     */
                           unsigned short PE5MD:2;      /*    PE5MD     */
                           unsigned short PE4MD:2;      /*    PE4MD     */
                           unsigned short PE3MD:2;      /*    PE3MD     */
                           unsigned short PE2MD:2;      /*    PE2MD     */
                           unsigned short PE1MD:2;      /*    PE1MD     */
                           unsigned short PE0MD:2;      /*    PE0MD     */
                           }       BIT;                 /*              */
                    }           PECR;                   /*              */
              union {                                   /* PFCR         */
                    unsigned short WORD;                /*  Word Access */
                    struct {                            /*  Bit  Access */
                           unsigned short PF7MD:2;      /*    PF7MD     */
                           unsigned short PF6MD:2;      /*    PF6MD     */
                           unsigned short PF5MD:2;      /*    PF5MD     */
                           unsigned short PF4MD:2;      /*    PF4MD     */
                           unsigned short PF3MD:2;      /*    PF3MD     */
                           unsigned short PF2MD:2;      /*    PF2MD     */
                           unsigned short PF1MD:2;      /*    PF1MD     */
                           unsigned short PF0MD:2;      /*    PF0MD     */
                           }       BIT;                 /*              */
                    }           PFCR;                   /*              */
              union {                                   /* PGCR         */
                    unsigned short WORD;                /*  Word Access */
                    struct {                            /*  Bit  Access */
                           unsigned short PG7MD:2;      /*    PG7MD     */
                           unsigned short PG6MD:2;      /*    PG6MD     */
                           unsigned short PG5MD:2;      /*    PG5MD     */
                           unsigned short PG4MD:2;      /*    PG4MD     */
                           unsigned short PG3MD:2;      /*    PG3MD     */
                           unsigned short PG2MD:2;      /*    PG2MD     */
                           unsigned short PG1MD:2;      /*    PG1MD     */
                           unsigned short PG0MD:2;      /*    PG0MD     */
                           }       BIT;                 /*              */
                    }           PGCR;                   /*              */
              union {                                   /* PHCR         */
                    unsigned short WORD;                /*  Word Access */
                    struct {                            /*  Bit  Access */
                           unsigned short      :2;      /*              */
                           unsigned short PH6MD:2;      /*    PH6MD     */
                           unsigned short PH5MD:2;      /*    PH5MD     */
                           unsigned short PH4MD:2;      /*    PH4MD     */
                           unsigned short PH3MD:2;      /*    PH3MD     */
                           unsigned short PH2MD:2;      /*    PH2MD     */
                           unsigned short PH1MD:2;      /*    PH1MD     */
                           unsigned short PH0MD:2;      /*    PH0MD     */
                           }       BIT;                 /*              */
                    }           PHCR;                   /*              */
              union {                                   /* PJCR         */
                    unsigned short WORD;                /*  Word Access */
                    struct {                            /*  Bit  Access */
                           unsigned short PJ7MD:2;      /*    PJ7MD     */
                           unsigned short PJ6MD:2;      /*    PJ6MD     */
                           unsigned short PJ5MD:2;      /*    PJ5MD     */
                           unsigned short PJ4MD:2;      /*    PJ4MD     */
                           unsigned short PJ3MD:2;      /*    PJ3MD     */
                           unsigned short PJ2MD:2;      /*    PJ2MD     */
                           unsigned short PJ1MD:2;      /*    PJ1MD     */
                           unsigned short PJ0MD:2;      /*    PJ0MD     */
                           }       BIT;                 /*              */
                    }           PJCR;                   /*              */
              union {                                   /* PKCR         */
                    unsigned short WORD;                /*  Word Access */
                    struct {                            /*  Bit  Access */
                           unsigned short PK7MD:2;      /*    PK7MD     */
                           unsigned short PK6MD:2;      /*    PK6MD     */
                           unsigned short PK5MD:2;      /*    PK5MD     */
                           unsigned short PK4MD:2;      /*    PK4MD     */
                           unsigned short PK3MD:2;      /*    PK3MD     */
                           unsigned short PK2MD:2;      /*    PK2MD     */
                           unsigned short PK1MD:2;      /*    PK1MD     */
                           unsigned short PK0MD:2;      /*    PK0MD     */
                           }       BIT;                 /*              */
                    }           PKCR;                   /*              */
              union {                                   /* PLCR         */
                    unsigned short WORD;                /*  Word Access */
                    struct {                            /*  Bit  Access */
                           unsigned short      :8;      /*              */
                           unsigned short PL3MD:2;      /*    PL3MD     */
                           unsigned short PL2MD:2;      /*    PL2MD     */
                           unsigned short PL1MD:2;      /*    PL1MD     */
                           unsigned short PL0MD:2;      /*    PL0MD     */
                           }       BIT;                 /*              */
                    }           PLCR;                   /*              */
              union {                                   /* SCPCR        */
                    unsigned short WORD;                /*  Word Access */
                    struct {                            /*  Bit  Access */
                           unsigned short       :4;     /*              */
                           unsigned short SCP5MD:2;     /*    SCP5MD    */
                           unsigned short SCP4MD:2;     /*    SCP4MD    */
                           unsigned short SCP3MD:2;     /*    SCP3MD    */
                           unsigned short SCP2MD:2;     /*    SCP2MD    */
                           unsigned short SCP1MD:2;     /*    SCP1MD    */
                           unsigned short SCP0MD:2;     /*    SCP0MD    */
                           }       BIT;                 /*              */
                    }           SCPCR;                  /*              */
              union {                                   /* PMCR         */
                    unsigned short WORD;                /*  Word Access */
                    struct {                            /*  Bit  Access */
                           unsigned short      :2;      /*              */
                           unsigned short PM6MD:2;      /*    PM6MD     */
                           unsigned short      :2;      /*              */
                           unsigned short PM4MD:2;      /*    PM4MD     */
                           unsigned short PM3MD:2;      /*    PM3MD     */
                           unsigned short PM2MD:2;      /*    PM2MD     */
                           unsigned short PM1MD:2;      /*    PM1MD     */
                           unsigned short PM0MD:2;      /*    PM0MD     */
                           }       BIT;                 /*              */
                    }           PMCR;                   /*              */
              union {                                   /* PNCR         */
                    unsigned short WORD;                /*  Word Access */
                    struct {                            /*  Bit  Access */
                           unsigned short PN7MD:2;      /*    PN7MD     */
                           unsigned short PN6MD:2;      /*    PN6MD     */
                           unsigned short PN5MD:2;      /*    PN5MD     */
                           unsigned short PN4MD:2;      /*    PN4MD     */
                           unsigned short PN3MD:2;      /*    PN3MD     */
                           unsigned short PN2MD:2;      /*    PN2MD     */
                           unsigned short PN1MD:2;      /*    PN1MD     */
                           unsigned short PN0MD:2;      /*    PN0MD     */
                           }       BIT;                 /*              */
                    }           PNCR;                   /*              */
              char              wk1[327724];            /*              */
              union {                                   /* PECR2        */
                    unsigned char BYTE;                 /*  Byte Access */
                    struct {                            /*  Bit  Access */
                           unsigned char      :1;       /*              */
                           unsigned char PE6MD:1;       /*    PE6MD     */
                           unsigned char PE5MD:1;       /*    PE5MD     */
                           unsigned char PE4MD:1;       /*    PE4MD     */
                           }       BIT;                 /*              */
                    }           PECR2;                  /*              */
              char              wk2;                    /*              */
              union {                                   /* PFCR2        */
                    unsigned char BYTE;                 /*  Byte Access */
                    struct {                            /*  Bit  Access */
                           unsigned char      :4;       /*              */
                           unsigned char PF3MD:1;       /*    PF3MD     */
                           unsigned char PF2MD:1;       /*    PF2MD     */
                           unsigned char PF1MD:1;       /*    PF1MD     */
                           unsigned char PF0MD:1;       /*    PF0MD     */
                           }       BIT;                 /*              */
                    }           PFCR2;                  /*              */
              char              wk3[15];                /*              */
              union {                                   /* PNCR2        */
                    unsigned char BYTE;                 /*  Byte Access */
                    struct {                            /*  Bit  Access */
                           unsigned char      :1;       /*              */
                           unsigned char PN6MD:1;       /*    PN6MD     */
                           unsigned char PN5MD:1;       /*    PN5MD     */
                           unsigned char PN4MD:1;       /*    PN4MD     */
                           unsigned char PN3MD:1;       /*    PN3MD     */
                           unsigned char PN2MD:1;       /*    PN2MD     */
                           unsigned char PN1MD:1;       /*    PN1MD     */
                           unsigned char PN0MD:1;       /*    PN0MD     */
                           }       BIT;                 /*              */
                    }           PNCR2;                  /*              */
};                                                      /*              */

#define CPG    (*(volatile struct st_cpg   *)0xFFFFFF80)/* CPG   Address*/
#define WDT    (*(volatile struct st_wdt   *)0xFFFFFF84)/* WDT   Address*/
#define RTC   (*(volatile struct st_rtc   *)0xA413FEC0) /* RTC	 Address*/
#define INTC   (*(volatile struct st_intc  *)0xFFFFFEE0)/* INTC  Address*/
#define INTX   (*(volatile struct st_intx  *)0xA4000000)/* INTX  Address*/
#define PA     (*(volatile struct st_pa    *)0xA4000120)/* PA    Address*/
#define PB     (*(volatile struct st_pb    *)0xA4000122)/* PB    Address*/
#define PC     (*(volatile struct st_pc    *)0xA4000124)/* PC    Address*/
#define PD     (*(volatile struct st_pd    *)0xA4000126)/* PD    Address*/
#define PE     (*(volatile struct st_pe    *)0xA4000128)/* PE    Address*/
#define PF     (*(volatile struct st_pf    *)0xA400012A)/* PF    Address*/
#define PG     (*(volatile struct st_pg    *)0xA400012C)/* PG    Address*/
#define PH     (*(volatile struct st_ph    *)0xA400012E)/* PH    Address*/
#define PJ     (*(volatile struct st_pj    *)0xA4000130)/* PJ    Address*/
#define PK     (*(volatile struct st_pk    *)0xA4000132)/* PK    Address*/
#define PL     (*(volatile struct st_pl    *)0xA4000134)/* PL    Address*/
#define SCP    (*(volatile struct st_scp   *)0xA4000136)/* SCP   Address*/
#define PM     (*(volatile struct st_pm    *)0xA4000138)/* PM    Address*/
#define PN     (*(volatile struct st_pn    *)0xA400013A)/* PN    Address*/
#define TMU    (*(volatile struct st_tmu   *)0xFFFFFE92)
#define TMU0   (*(volatile struct st_tmu0  *)0xFFFFFE94)
#define TMU1   (*(volatile struct st_tmu0  *)0xFFFFFEA0)
#define TMU2   (*(volatile struct st_tmu2  *)0xFFFFFEAC)
#define USB    (*(volatile struct st_usb   *)0xA40A0008)/* UBC   Address*/
#define SCIF0  (*(volatile struct st_scif  *)0xA4400000)
#define SCIF2  (*(volatile struct st_scif  *)0xA4410000)
#define STBCR3 (*(volatile union  un_stbcr3*)0xA40A0000)/* STBCR3Address*/
#define PFC    (*(volatile struct st_pfc   *)0xA4000100)/* PFC   Address*/

void Init_SerialPort()
{
	int i;
	STBCR3.BYTE = 0xFD;

	if (SCIF2.SCSSR.BIT.ORER == 1) SCIF2.SCSSR.BIT.ORER = 0;
	if (SCIF2.SCSSR.BIT.ER == 1) SCIF2.SCSSR.BIT.ER = 0;
	if (SCIF2.SCSSR.BIT.BRK == 1) SCIF2.SCSSR.BIT.BRK = 0;
	if (SCIF2.SCSSR.BIT.FER == 1) SCIF2.SCSSR.BIT.FER = 0;
	if (SCIF2.SCSSR.BIT.RDF == 1) SCIF2.SCSSR.BIT.RDF = 0;
	if (SCIF2.SCSSR.BIT.DR == 1) SCIF2.SCSSR.BIT.DR = 0;

	SCIF2.SCFER.WORD = 0x00;

	SCIF2.SCSSR.BIT.RDF = 0;
	SCIF2.SCSCR.BIT.TE = SCIF2.SCSCR.BIT.RE = 0;
	SCIF2.SCFCR.BIT.RFRST = SCIF2.SCFCR.BIT.TFRST = 1;
	SCIF2.SCSSR.BIT.ER = SCIF2.SCSSR.BIT.DR = SCIF2.SCSSR.BIT.BRK = 0;
	SCIF2.SCSCR.BIT.CKE = 0;
	SCIF2.SCFCR.BIT.TSE = SCIF2.SCSCR.BIT.TSIE = SCIF2.SCSCR.BIT.ERIE = SCIF2.SCSCR.BIT.BRIE = SCIF2.SCSCR.BIT.DRIE = SCIF2.SCSCR.BIT.TIE = SCIF2.SCSCR.BIT.RIE = 0;

	SCIF2.SCSMR.BIT.SRC = 0;
	SCIF2.SCSMR.BIT.CA = 0;
	SCIF2.SCSMR.BIT.CHR = 0;
	SCIF2.SCSMR.BIT._PE = 0;
	SCIF2.SCSMR.BIT.STOP = 1;
	SCIF2.SCSMR.BIT.CKS = 0;

	SCIF2.SCBRR = 95;
	
	for (i=0; i<8192; i++);

	Setup_Serial();
	
	SCIF2.SCFCR.BIT.RTRG = 0;
	SCIF2.SCFCR.BIT.TTRG = 3;
	SCIF2.SCFCR.BIT.MCE = 0;
	SCIF2.SCFCR.BIT.LOOP = 0;
	SCIF2.SCFCR.BIT.RFRST = SCIF2.SCFCR.BIT.TFRST = 0;
	SCIF2.SCSCR.BIT.TE = SCIF2.SCSCR.BIT.RE = 1;
	SCIF2.SCSCR.BIT.TIE = SCIF2.SCSCR.BIT.RIE = 0;
	
	SCIF2.SCTDSR = 0;
}

void Start_SerialPort()
{
	SCIF2.SCSSR.BIT.RDF = 0;
	SCIF2.SCSSR.BIT.TDFE = 0;
}

void SerialTransmit(unsigned char value)
{
	int i;
	while (SCIF2.SCSSR.BIT.TDFE != 1);
	SCIF2.SCFTDR = value;
	for (i=0; i<8192; i++);
	SCIF2.SCSSR.BIT.TEND = SCIF2.SCSSR.BIT.TDFE = 0;
}

unsigned char SerialReceive()
{
	int i;
	if (SCIF2.SCSSR.BIT.RDF == 1) {
		i = SCIF2.SCFRDR;
		SCIF2.SCSSR.BIT.RDF = 0;
		return i;
	}
}

void Stop_SerialPort()
{
	SCIF2.SCSCR.BIT.TE = SCIF2.SCSCR.BIT.RE = 0;
}

void RTCReset()
{
	RTC.RCR2.BIT.RESET = 1;
}

void RTCStart()
{
	RTC.RCR2.BIT.START = 1;
}

void RTCStop()
{
	RTC.RCR2.BIT.START = 0;
}

void RTCSetYear(unsigned char thousands, unsigned char hundreds, unsigned char tens, unsigned char ones)
{
	RTC.RYRCNT.BIT.Y1000 = thousands;
	RTC.RYRCNT.BIT.Y100 = hundreds;
	RTC.RYRCNT.BIT.Y10 = tens;
	RTC.RYRCNT.BIT.Y1 = ones;
}

void RTCSetMonth(unsigned char tens, unsigned char ones)
{
	RTC.RMONCNT.BIT.M10 = tens;
	RTC.RMONCNT.BIT.M1 = ones;
}

void RTCSetDayOfMonth(unsigned char tens, unsigned char ones)
{
	RTC.RDAYCNT.BIT.D10 = tens;
	RTC.RDAYCNT.BIT.D1 = ones;
}

void RTCSetDayOfWeek(unsigned char ones)
{
	RTC.RWKCNT.BYTE = ones;
}

void RTCSetHour(unsigned char tens, unsigned char ones)
{
	RTC.RHRCNT.BIT.H10 = tens;
	RTC.RHRCNT.BIT.H1 = ones;
}

void RTCSetMinute(unsigned char tens, unsigned char ones)
{
	RTC.RMINCNT.BIT.M10 = tens;
	RTC.RMINCNT.BIT.M1 = ones;
}

void RTCSetSecond(unsigned char tens, unsigned char ones)
{
	RTC.RSECCNT.BIT.S10 = tens;
	RTC.RSECCNT.BIT.S1 = ones;
}

int RTCReadYear()
{
	int year;
	RTC.RCR1.BIT.CIE = 0;
	RTC.RCR1.BIT.CF = 0;
	year = (RTC.RYRCNT.BIT.Y1000 * 1000) + (RTC.RYRCNT.BIT.Y100 * 100) + (RTC.RYRCNT.BIT.Y10 * 10) + RTC.RYRCNT.BIT.Y1;
	return year;
}

int RTCReadMonth()
{
	int month;
	RTC.RCR1.BIT.CIE = 0;
	RTC.RCR1.BIT.CF = 0;
	month = (RTC.RMONCNT.BIT.M10 * 10) + RTC.RMONCNT.BIT.M1;
	return month;
}

int RTCReadDayOfMonth()
{
	int monthday;
	RTC.RCR1.BIT.CIE = 0;
	RTC.RCR1.BIT.CF = 0;
	monthday = (RTC.RDAYCNT.BIT.D10 * 10) + RTC.RDAYCNT.BIT.D1;
	return monthday;
}

int RTCReadDayOfWeek()
{
	int day;
	RTC.RCR1.BIT.CIE = 0;
	RTC.RCR1.BIT.CF = 0;
	day = RTC.RWKCNT.BYTE;
	return day;
}

int RTCReadHour()
{
	int hour;
	RTC.RCR1.BIT.CIE = 0;
	RTC.RCR1.BIT.CF = 0;
	hour = (RTC.RHRCNT.BIT.H10 * 10) + RTC.RHRCNT.BIT.H1;
	return hour;
}

int RTCReadMinute()
{
	int minute;
	RTC.RCR1.BIT.CIE = 0;
	RTC.RCR1.BIT.CF = 0;
	minute = (RTC.RMINCNT.BIT.M10 * 10) + RTC.RMINCNT.BIT.M1;
	return minute;
}

int RTCReadSecond()
{
	int second;
	RTC.RCR1.BIT.CIE = 0;
	RTC.RCR1.BIT.CF = 0;
	second = (RTC.RSECCNT.BIT.S10 * 10) + RTC.RSECCNT.BIT.S1;
	return second;
}

unsigned char ReadPort_A()
{
	unsigned char port_a;
	port_a = PA.DR.BYTE;
	return port_a;
}

unsigned char ReadPort_B()
{
	unsigned char port_b;
	port_b = PB.DR.BYTE;	
	return port_b;
}

unsigned char ReadPort_C()
{
	unsigned char port_c;	
	port_c = PC.DR.BYTE;
	return port_c;
}

unsigned char ReadPort_D()
{
	unsigned char port_d;
	port_d = PD.DR.BYTE;
	return port_d;
}

unsigned char ReadPort_E()
{
	unsigned char port_e;
	port_e = PE.DR.BYTE;
	return port_e;
}

unsigned char ReadPort_F()
{
	unsigned char port_f;
	port_f = PF.DR.BYTE;
	return port_f;
}

unsigned char ReadPort_G()
{
	unsigned char port_g;
	port_g = PG.DR.BYTE;
	return port_g;
}

unsigned char ReadPort_H()
{
	unsigned char port_h;
	port_h = PH.DR.BYTE;
	return port_h;
}

unsigned char ReadPort_J()
{
	unsigned char port_j;
	port_j = PJ.DR.BYTE;
	return port_j;
}

unsigned char ReadPort_K()
{
	unsigned char port_k;
	port_k = PK.DR.BYTE;
	return port_k;
}

unsigned char ReadPort_L()
{
	unsigned char port_l;
	port_l = PL.DR.BYTE;
	return port_l;
}

unsigned char ReadPort_M()
{
	unsigned char port_m;
	port_m = PM.DR.BYTE;
	return port_m;
}

unsigned char ReadPort_N()
{
	unsigned char port_n;
	port_n = PN.DR.BYTE;
	return port_n;
}

unsigned char ReadPort_SCP()
{
	unsigned char port_scp;
	port_scp = SCP.DR.BYTE;
	return port_scp;
}

typedef struct {
	char which;
	unsigned long *buffer_one;
	unsigned long *buffer_two;
	unsigned long *buffer_three;
	int countdown_buffer1;
	int countdown_buffer2;
	int countdown_buffer3;
} GRAY_SETUP;

GRAY_SETUP gray_test;

#pragma interrupt(GrayInterrupt)

void GrayInterrupt(void)
{
	if(gray_test.which == 0)
	{
		DrawAll(gray_test.buffer_one);
		gray_test.which = 1;
		TMU.TSTR.BIT.STR0 = 0;
		TMU0.TCNT = TMU0.TCOR = gray_test.countdown_buffer2;
	} else {
		DrawAll(gray_test.buffer_two);
		gray_test.which = 0;
		TMU.TSTR.BIT.STR0 = 0;
		TMU0.TCNT = TMU0.TCOR = gray_test.countdown_buffer1;
	}
	TMU0.TCR.BIT.UNF = 0;
	TMU.TSTR.BIT.STR0 = 1;
}

#pragma interrupt(GrayInterrupt3)

void GrayInterrupt3(void)
{
	if(gray_test.which == 0)
	{
		DrawAll(gray_test.buffer_one);
		gray_test.which = 1;
		TMU.TSTR.BIT.STR0 = 0;
		TMU0.TCNT = TMU0.TCOR = gray_test.countdown_buffer2;
	} else if (gray_test.which == 1) {
		DrawAll(gray_test.buffer_two);
		gray_test.which = 2;
		TMU.TSTR.BIT.STR0 = 0;
		TMU0.TCNT = TMU0.TCOR = gray_test.countdown_buffer3;
	} else if (gray_test.which == 2) {
		DrawAll(gray_test.buffer_three);
		gray_test.which = 2;
		TMU.TSTR.BIT.STR0 = 0;
		TMU0.TCNT = TMU0.TCOR = gray_test.countdown_buffer1;
	}
	TMU0.TCR.BIT.UNF = 0;
	TMU.TSTR.BIT.STR0 = 1;
}

void GrayInit(int countdown, int countdown2)
{
	set_vbr((void *)((unsigned int)GrayInterrupt - INT_OFFSET));
	TMU0.TCR.WORD = 0x0022;
	gray_test.countdown_buffer1 = countdown;
	gray_test.countdown_buffer2 = countdown2;
	TMU0.TCNT = TMU0.TCOR = gray_test.countdown_buffer1;
	INTC.IPRA.WORD = 0xf000 ; 
	INTC.IPRB.WORD = 0x0000 ;
	INTX.IPRC.WORD = 0x0000 ;  
	INTX.IPRD.WORD = 0x0000 ; 
	INTX.IPRE.WORD = 0x0000 ;
	INTX.IPRF.WORD = 0x0000 ; 
	INTX.IPRG.WORD = 0x0000 ;
	INTX.IPRH.WORD = 0x0000 ;   
	TMU.TSTR.BIT.STR0 = 1;
}

void GrayInit3(int countdown, int countdown2, int countdown3)
{
	set_vbr((void *)((unsigned int)GrayInterrupt3 - INT_OFFSET));
	TMU0.TCR.WORD = 0x0022;
	gray_test.countdown_buffer1 = countdown;
	gray_test.countdown_buffer2 = countdown2;
	gray_test.countdown_buffer3 = countdown3;
	TMU0.TCNT = TMU0.TCOR = gray_test.countdown_buffer1;
	INTC.IPRA.WORD = 0xf000 ; 
	INTC.IPRB.WORD = 0x0000 ;
	INTX.IPRC.WORD = 0x0000 ;  
	INTX.IPRD.WORD = 0x0000 ; 
	INTX.IPRE.WORD = 0x0000 ;
	INTX.IPRF.WORD = 0x0000 ; 
	INTX.IPRG.WORD = 0x0000 ;
	INTX.IPRH.WORD = 0x0000 ;   
	TMU.TSTR.BIT.STR0 = 1;
}

void GrayLinkBuffers(unsigned long *buffer1, unsigned long *buffer2)
{
	gray_test.buffer_one = buffer1;
	gray_test.buffer_two = buffer2;
}

void GrayLinkBuffers3(unsigned long *buffer1, unsigned long *buffer2, unsigned long *buffer3)
{
	gray_test.buffer_one = buffer1;
	gray_test.buffer_two = buffer2;
	gray_test.buffer_three = buffer3;
}

void GrayEnd()
{
	TMU0.TCR.WORD = 0x0000;
	TMU0.TCNT = TMU0.TCOR = 0xFFFFFFFF;
	INTC.IPRA.WORD = 0x0000;  
	TMU.TSTR.BIT.STR0 = 0;
}
	
void CPUSpeedNormal()
{
	WDT.WTCSR.WRITE = 96;
	WDT.WTCNT.WRITE = 0;
	CPG.FRQCR.BIT._PFC = 1;
	CPG.FRQCR.BIT.STC = 0;
}

void CPUSpeedDouble()
{
	WDT.WTCSR.WRITE = 96;
	WDT.WTCNT.WRITE = 0;
	CPG.FRQCR.BIT._PFC = 1;
	CPG.FRQCR.BIT.STC = 1;
}

void CPUSpeedTriple()
{
	WDT.WTCSR.WRITE = 96;
	WDT.WTCNT.WRITE = 0;
	CPG.FRQCR.BIT._PFC = 2;
	CPG.FRQCR.BIT.STC = 2;
}

void CPUSpeedQuadruple()
{
	WDT.WTCSR.WRITE = 96;
	WDT.WTCNT.WRITE = 0;
	CPG.FRQCR.BIT._PFC = 3;
	CPG.FRQCR.BIT.STC = 3;
}

int string_input(int x, int y, char string[])
{
	unsigned int key;
	int displacement = 0;
	int set_alpha = 0;
	
	PrintMini(x, y, (unsigned char*)"_", MINI_OVER);

	while((key != KEY_CTRL_EXE) && (displacement != 25))
	{
		GetKey(&key);
		if(key == KEY_CTRL_AC)
		{
			PrintMini(0, y, (unsigned char*)"                                              ", MINI_OVER);
			memset(string, 0, 42);
			displacement = 0;
			PrintMini(x, y, (unsigned char*)"_", MINI_OVER);
			GetKey(&key);
		}
		if(key == KEY_CHAR_LPAR)
		{
			PrintMini(x + (displacement*5), y, (unsigned char*)"(", MINI_OVER);
			string[displacement] = '(';
			displacement++;
		}
		if(key == KEY_CHAR_RPAR)
		{
			PrintMini(x + (displacement*5), y, (unsigned char*)")", MINI_OVER);
			string[displacement] = ')';
			displacement++;
		}
		if(key == KEY_CHAR_0)
		{
			PrintMini(x + (displacement*5), y, (unsigned char*)"0", MINI_OVER);
			string[displacement] = '0';
			displacement++;
		}
		if(key == KEY_CHAR_1)
		{
			PrintMini(x + (displacement*5), y, (unsigned char*)"1", MINI_OVER);
			string[displacement] = '1';
			displacement++;
		}
		if(key == KEY_CHAR_2)
		{
			PrintMini(x + (displacement*5), y, (unsigned char*)"2", MINI_OVER);
			string[displacement] = '2';
			displacement++;
		}
		if(key == KEY_CHAR_3)
		{
			PrintMini(x + (displacement*5), y, (unsigned char*)"3", MINI_OVER);
			string[displacement] = '3';
			displacement++;
		}
		if(key == KEY_CHAR_4)
		{
			PrintMini(x + (displacement*5), y, (unsigned char*)"4", MINI_OVER);
			string[displacement] = '4';
			displacement++;
		}
		if(key == KEY_CHAR_5)
		{
			PrintMini(x + (displacement*5), y, (unsigned char*)"5", MINI_OVER);
			string[displacement] = '5';
			displacement++;
		}
		if(key == KEY_CHAR_6)
		{
			PrintMini(x + (displacement*5), y, (unsigned char*)"6", MINI_OVER);
			string[displacement] = '6';
			displacement++;
		}
		if(key == KEY_CHAR_7)
		{
			PrintMini(x + (displacement*5), y, (unsigned char*)"7", MINI_OVER);
			string[displacement] = '7';
			displacement++;
		}
		if(key == KEY_CHAR_8)
		{
			PrintMini(x + (displacement*5), y, (unsigned char*)"8", MINI_OVER);
			string[displacement] = '8';
			displacement++;
		}
		if(key == KEY_CHAR_9)
		{
			PrintMini(x + (displacement*5), y, (unsigned char*)"9", MINI_OVER);
			string[displacement] = '9';
			displacement++;
		}
		if(key == KEY_CHAR_COMMA)
		{
			PrintMini(x + (displacement*5), y, (unsigned char*)",", MINI_OVER);
			string[displacement] = ' ';
			displacement++;
		}
		if(key == KEY_CHAR_DP)
		{
			PrintMini(x + (displacement*5), y, (unsigned char*)".", MINI_OVER);
			string[displacement] = '.';
			displacement++;
		}
		if(key == KEY_CHAR_PLUS)
		{
			PrintMini(x + (displacement*5), y, (unsigned char*)"+", MINI_OVER);
			string[displacement] = '+';
			displacement++;
		}
		if(key == KEY_CHAR_MINUS)
		{
			PrintMini(x + (displacement*5), y, (unsigned char*)"-", MINI_OVER);
			string[displacement] = '-';
			displacement++;
		}
		if(key == KEY_CHAR_MULT)
		{
			PrintMini(x + (displacement*5), y, (unsigned char*)"*", MINI_OVER);
			string[displacement] = '*';
			displacement++;
		}
		if(key == KEY_CHAR_DIV)
		{
			PrintMini(x + (displacement*5), y, (unsigned char*)"/", MINI_OVER);
			string[displacement] = '/';
			displacement++;
		}
		if(key == KEY_CTRL_SHIFT)
		{
			if(set_alpha == 1)
			{
				set_alpha = 0;
			} else {
				set_alpha = 1;
			}
		}
		if(key == KEY_CTRL_ALPHA)
		{
			GetKey(&key);
		}
		if(key == KEY_CTRL_AC)
		{
			PrintMini(0, y, (unsigned char*)"                                                 ", MINI_OVER);
			memset(string, 0, 42);
			displacement = 0;
			PrintMini(x, y, (unsigned char*)"_", MINI_OVER);
		}
		if(key == KEY_CHAR_A) {
			if(set_alpha == 1)
			{
				PrintMini(x + (displacement*5), y, (unsigned char*)"A", MINI_OVER);
				string[displacement] = 'A';
				displacement++;
			} else {
				PrintMini(x + (displacement*5), y, (unsigned char*)"a", MINI_OVER);
				string[displacement] = 'a';
				displacement++;
			}
		}
		if(key == KEY_CHAR_B) {
			if(set_alpha == 1)
			{
				PrintMini(x + (displacement*5), y, (unsigned char*)"B", MINI_OVER);
				string[displacement] = 'B';
				displacement++;
			} else {
				PrintMini(x + (displacement*5), y, (unsigned char*)"b", MINI_OVER);
				string[displacement] = 'b';
				displacement++;
			}
		}
		if(key == KEY_CHAR_C) {
			if(set_alpha == 1)
			{
				PrintMini(x + (displacement*5), y, (unsigned char*)"C", MINI_OVER);
				string[displacement] = 'D';
				displacement++;
			} else {
				PrintMini(x + (displacement*5), y, (unsigned char*)"c", MINI_OVER);
				string[displacement] = 'c';
				displacement++;
			}
		}
		if(key == KEY_CHAR_D) {
			if(set_alpha == 1)
			{
				PrintMini(x + (displacement*5), y, (unsigned char*)"D", MINI_OVER);
				string[displacement] = 'D';
				displacement++;
			} else {
				PrintMini(x + (displacement*5), y, (unsigned char*)"d", MINI_OVER);
				string[displacement] = 'd';
				displacement++;
			}
		}
		if(key == KEY_CHAR_E) {
			if(set_alpha == 1)
			{
				PrintMini(x + (displacement*5), y, (unsigned char*)"E", MINI_OVER);
				string[displacement] = 'E';
				displacement++;
			} else {
				PrintMini(x + (displacement*5), y, (unsigned char*)"e", MINI_OVER);
				string[displacement] = 'e';
				displacement++;
			}
		}
		if(key == KEY_CHAR_F) {
			if(set_alpha == 1)
			{
				PrintMini(x + (displacement*5), y, (unsigned char*)"F", MINI_OVER);
				string[displacement] = 'F';
				displacement++;
			} else {
				PrintMini(x + (displacement*5), y, (unsigned char*)"f", MINI_OVER);
				string[displacement] = 'f';
				displacement++;
			}
		}
		if(key == KEY_CHAR_G) {
			if(set_alpha == 1)
			{
				PrintMini(x + (displacement*5), y, (unsigned char*)"G", MINI_OVER);
				string[displacement] = 'G';
				displacement++;
			} else {
				PrintMini(x + (displacement*5), y, (unsigned char*)"g", MINI_OVER);
				string[displacement] = 'g';
				displacement++;
			}
		}
		if(key == KEY_CHAR_H) {
			if(set_alpha == 1)
			{
				PrintMini(x + (displacement*5), y, (unsigned char*)"H", MINI_OVER);
				string[displacement] = 'H';
				displacement++;
			} else {
				PrintMini(x + (displacement*5), y, (unsigned char*)"h", MINI_OVER);
				string[displacement] = 'h';
				displacement++;
			}
		}
		if(key == KEY_CHAR_I) {
			if(set_alpha == 1)
			{
				PrintMini(x + (displacement*5), y, (unsigned char*)"I", MINI_OVER);
				string[displacement] = 'I';
				displacement++;
			} else {
				PrintMini(x + (displacement*5), y, (unsigned char*)"i", MINI_OVER);
				string[displacement] = 'i';
				displacement++;
			}
		}
		if(key == KEY_CHAR_J) {
			if(set_alpha == 1)
			{
				PrintMini(x + (displacement*5), y, (unsigned char*)"J", MINI_OVER);
				string[displacement] = 'J';
				displacement++;
			} else {
				PrintMini(x + (displacement*5), y, (unsigned char*)"j", MINI_OVER);
				string[displacement] = 'j';
				displacement++;
			}
		}
		if(key == KEY_CHAR_K) {
			if(set_alpha == 1)
			{
				PrintMini(x + (displacement*5), y, (unsigned char*)"K", MINI_OVER);
				string[displacement] = 'K';
				displacement++;
			} else {
				PrintMini(x + (displacement*5), y, (unsigned char*)"k", MINI_OVER);
				string[displacement] = 'k';
				displacement++;
			}
		}
		if(key == KEY_CHAR_L) {
			if(set_alpha == 1)
			{
				PrintMini(x + (displacement*5), y, (unsigned char*)"L", MINI_OVER);
				string[displacement] = 'L';
				displacement++;
			} else {
				PrintMini(x + (displacement*5), y, (unsigned char*)"l", MINI_OVER);
				string[displacement] = 'l';
				displacement++;
			}
		}
		if(key == KEY_CHAR_M) {
			if(set_alpha == 1)
			{
				PrintMini(x + (displacement*5), y, (unsigned char*)"M", MINI_OVER);
				string[displacement] = 'M';
				displacement++;
			} else {
				PrintMini(x + (displacement*5), y, (unsigned char*)"m", MINI_OVER);
				string[displacement] = 'm';
				displacement++;
			}
		}
		if(key == KEY_CHAR_N) {
			if(set_alpha == 1)
			{
				PrintMini(x + (displacement*5), y, (unsigned char*)"N", MINI_OVER);
				string[displacement] = 'N';
				displacement++;
			} else {
				PrintMini(x + (displacement*5), y, (unsigned char*)"n", MINI_OVER);
				string[displacement] = 'n';
				displacement++;
			}
		}
		if(key == KEY_CHAR_O) {
			if(set_alpha == 1)
			{
				PrintMini(x + (displacement*5), y, (unsigned char*)"O", MINI_OVER);
				string[displacement] = 'O';
				displacement++;
			} else {
				PrintMini(x + (displacement*5), y, (unsigned char*)"o", MINI_OVER);
				string[displacement] = 'o';
				displacement++;
			}
		}
		if(key == KEY_CHAR_P) {
			if(set_alpha == 1)
			{
				PrintMini(x + (displacement*5), y, (unsigned char*)"P", MINI_OVER);
				string[displacement] = 'P';
				displacement++;
			} else {
				PrintMini(x + (displacement*5), y, (unsigned char*)"p", MINI_OVER);
				string[displacement] = 'p';
				displacement++;
			}
		}
		if(key == KEY_CHAR_Q) {
			if(set_alpha == 1)
			{
				PrintMini(x + (displacement*5), y, (unsigned char*)"Q", MINI_OVER);
				string[displacement] = 'Q';
				displacement++;
			} else {
				PrintMini(x + (displacement*5), y, (unsigned char*)"q", MINI_OVER);
				string[displacement] = 'q';
				displacement++;
			}
		}
		if(key == KEY_CHAR_R) {
			if(set_alpha == 1)
			{
				PrintMini(x + (displacement*5), y, (unsigned char*)"R", MINI_OVER);
				string[displacement] = 'R';
				displacement++;
			} else {
				PrintMini(x + (displacement*5), y, (unsigned char*)"r", MINI_OVER);
				string[displacement] = 'r';
				displacement++;
			}
		}
		if(key == KEY_CHAR_S) {
			if(set_alpha == 1)
			{
				PrintMini(x + (displacement*5), y, (unsigned char*)"S", MINI_OVER);
				string[displacement] = 'S';
				displacement++;
			} else {
				PrintMini(x + (displacement*5), y, (unsigned char*)"s", MINI_OVER);
				string[displacement] = 's';
				displacement++;
			}
		}
		if(key == KEY_CHAR_T) {
			if(set_alpha == 1)
			{
				PrintMini(x + (displacement*5), y, (unsigned char*)"T", MINI_OVER);
				string[displacement] = 'T';
				displacement++;
			} else {
				PrintMini(x + (displacement*5), y, (unsigned char*)"t", MINI_OVER);
				string[displacement] = 't';
				displacement++;
			}
		}
		if(key == KEY_CHAR_U) {
			if(set_alpha == 1)
			{
				PrintMini(x + (displacement*5), y, (unsigned char*)"U", MINI_OVER);
				string[displacement] = 'U';
				displacement++;
			} else {
				PrintMini(x + (displacement*5), y, (unsigned char*)"u", MINI_OVER);
				string[displacement] = 'u';
				displacement++;
			}
		}
		if(key == KEY_CHAR_V) {
			if(set_alpha == 1)
			{
				PrintMini(x + (displacement*5), y, (unsigned char*)"V", MINI_OVER);
				string[displacement] = 'V';
				displacement++;
			} else {
				PrintMini(x + (displacement*5), y, (unsigned char*)"v", MINI_OVER);
				string[displacement] = 'v';
				displacement++;
			}
		}
		if(key == KEY_CHAR_W) {
			if(set_alpha == 1)
			{
				PrintMini(x + (displacement*5), y, (unsigned char*)"W", MINI_OVER);
				string[displacement] = 'W';
				displacement++;
			} else {
				PrintMini(x + (displacement*5), y, (unsigned char*)"w", MINI_OVER);
				string[displacement] = 'w';
				displacement++;
			}
		}
		if(key == KEY_CHAR_X) {
			if(set_alpha == 1)
			{
				PrintMini(x + (displacement*5), y, (unsigned char*)"X", MINI_OVER);
				string[displacement] = 'X';
				displacement++;
			} else {
				PrintMini(x + (displacement*5), y, (unsigned char*)"x", MINI_OVER);
				string[displacement] = 'x';
				displacement++;
			}
		}
		if(key == KEY_CHAR_Y) {
			if(set_alpha == 1)
			{
				PrintMini(x + (displacement*5), y, (unsigned char*)"Y", MINI_OVER);
				string[displacement] = 'Y';
				displacement++;
			} else {
				PrintMini(x + (displacement*5), y, (unsigned char*)"y", MINI_OVER);
				string[displacement] = 'y';
				displacement++;
			}
		}
		if(key == KEY_CHAR_Z) {
			if(set_alpha == 1)
			{
				PrintMini(x + (displacement*5), y, (unsigned char*)"Z", MINI_OVER);
				string[displacement] = 'Z';
				displacement++;
			} else {
				PrintMini(x + (displacement*5), y, (unsigned char*)"z", MINI_OVER);
				string[displacement] = 'z';
				displacement++;
			}
		}
		if(key == KEY_CHAR_SPACE) {
			PrintMini(x + (displacement*5), y, (unsigned char*)" ", MINI_OVER);
			string[displacement] = ' ';
			displacement++;
		}
		PrintMini(x + (displacement*5), y, (unsigned char*)"_", MINI_OVER); 
	}
	string[displacement] = '\0';
	return 1;
}

char GetPixel(char x, char y, unsigned char *buffer)
{
	int i;
	i=((int)x >> 3) + ((int)y << 4) ;   
	return (buffer[i] &(0x80 >> (x & 7)) ? 1 : 0);
}

void PlotPixel(char x, char y, unsigned char *buffer, char draw_type)
{
	int i=((int)x >> 3) + ((int)y << 4) ;  
 	if (draw_type==1) buffer[i] = buffer[i] | 1<<(7-(x & 0x7)); 
  	if (draw_type==0) buffer[i] = buffer[i] & ~(1<<(7-(x & 0x7))); 
  	if (draw_type==2) buffer[i] = buffer[i] ^ (1<<(7-(x & 0x7)));
}

/* A C Implementation of the Bresenham Line Drawing Algorithm */

void DrawLine(char x1, char y1, char x2, char y2, unsigned char *buffer, char color)
{
	int i, delta_x, delta_y, sdelta_x, sdelta_y, absdelta_x, absdelta_y;
	int x, y, plot_x, plot_y;

	delta_x = x2 - x1;
	delta_y = y2 - y1;
	absdelta_x = abs(delta_x);
	absdelta_y = abs(delta_y);
	sdelta_x = sgn(delta_x);
	sdelta_y = sgn(delta_y);
	x = absdelta_y>>1;
  	y = absdelta_x>>1;
	plot_x = x1;
	plot_y = y1;

	if (absdelta_x >= absdelta_y) {
		for(i = 0; i < absdelta_x; i++) {
			y += absdelta_y;
			if (y >= absdelta_x) {
				y -= absdelta_x;
				plot_y += sdelta_y;
			}
			plot_x += sdelta_x;
			PlotPixel(plot_x, plot_y, buffer, color);
		}
	} else {
		for(i = 0; i < absdelta_y; i++) {
			x += absdelta_x;
			if (x >= absdelta_y) {
				x -= absdelta_y;
				plot_x += sdelta_x;
			}
			plot_x += sdelta_y;
			PlotPixel(plot_x, plot_y, buffer, color);
		}
	}
}

/* Transparency routines added thanks to Bebe-Vador */

void DrawSprite8(char x, char y, unsigned char *buffer, char *bitmap, int trans)  
{
	int j, v1;
	for (j=0;j<8;j++)
	{
		v1 = bitmap[j];
		if( (v1 & 0x80)==0x80 || (trans==0) ) PlotPixel(x,   y+j, buffer, (v1 & 0x80)==0x80);
		if( (v1 & 0x40)==0x40 || (trans==0) ) PlotPixel(x+1, y+j, buffer, (v1 & 0x40)==0x40); 
		if( (v1 & 0x20)==0x20 || (trans==0) ) PlotPixel(x+2, y+j, buffer, (v1 & 0x20)==0x20);
		if( (v1 & 0x10)==0x10 || (trans==0) ) PlotPixel(x+3, y+j, buffer, (v1 & 0x10)==0x10);
		if( (v1 & 0x08)==0x08 || (trans==0) ) PlotPixel(x+4, y+j, buffer, (v1 & 0x08)==0x08);
		if( (v1 & 0x04)==0x04 || (trans==0) ) PlotPixel(x+5, y+j, buffer, (v1 & 0x04)==0x04);
		if( (v1 & 0x02)==0x02 || (trans==0) ) PlotPixel(x+6, y+j, buffer, (v1 & 0x02)==0x02);
		if( (v1 & 0x01)==0x01 || (trans==0) ) PlotPixel(x+7, y+j, buffer, (v1 & 0x01)==0x01);
	}
}

void DrawSprite8x16(char x, char y, unsigned char *buffer, char *bitmap, int trans)  
{
	int j, v1;
	for (j=0;j<16;j++)
	{
		v1 = bitmap[j];
		if( (v1 & 0x80)==0x80 || (trans==0) ) PlotPixel(x,   y+j, buffer, (v1 & 0x80)==0x80);
		if( (v1 & 0x40)==0x40 || (trans==0) ) PlotPixel(x+1, y+j, buffer, (v1 & 0x40)==0x40); 
		if( (v1 & 0x20)==0x20 || (trans==0) ) PlotPixel(x+2, y+j, buffer, (v1 & 0x20)==0x20);
		if( (v1 & 0x10)==0x10 || (trans==0) ) PlotPixel(x+3, y+j, buffer, (v1 & 0x10)==0x10);
		if( (v1 & 0x08)==0x08 || (trans==0) ) PlotPixel(x+4, y+j, buffer, (v1 & 0x08)==0x08);
		if( (v1 & 0x04)==0x04 || (trans==0) ) PlotPixel(x+5, y+j, buffer, (v1 & 0x04)==0x04);
		if( (v1 & 0x02)==0x02 || (trans==0) ) PlotPixel(x+6, y+j, buffer, (v1 & 0x02)==0x02);
		if( (v1 & 0x01)==0x01 || (trans==0) ) PlotPixel(x+7, y+j, buffer, (v1 & 0x01)==0x01);
	}
}

void DrawSprite16(char x, char y, unsigned char *buffer, char *bitmap, int trans)  
{
	int j, v1, i, k;
	for (j=0;j<32;j++)
	{
		v1 = bitmap[j];
		i = y+(j/2);
		if( (v1 & 0x80)==0x80 || (trans==0) ) PlotPixel(x, i, buffer, (v1 & 0x80)==0x80);
		if( (v1 & 0x40)==0x40 || (trans==0) ) PlotPixel(x+1, i, buffer, (v1 & 0x40)==0x40); 
		if( (v1 & 0x20)==0x20 || (trans==0) ) PlotPixel(x+2, i, buffer, (v1 & 0x20)==0x20);
		if( (v1 & 0x10)==0x10 || (trans==0) ) PlotPixel(x+3, i, buffer, (v1 & 0x10)==0x10);
		if( (v1 & 0x08)==0x08 || (trans==0) ) PlotPixel(x+4, i, buffer, (v1 & 0x08)==0x08);
		if( (v1 & 0x04)==0x04 || (trans==0) ) PlotPixel(x+5, i, buffer, (v1 & 0x04)==0x04);
		if( (v1 & 0x02)==0x02 || (trans==0) ) PlotPixel(x+6, i, buffer, (v1 & 0x02)==0x02);
		if( (v1 & 0x01)==0x01 || (trans==0) ) PlotPixel(x+7, i, buffer, (v1 & 0x01)==0x01);
		v1 = bitmap[j+1];
		if( (v1 & 0x80)==0x80 || (trans==0) ) PlotPixel(x+8, i, buffer, (v1 & 0x80)==0x80);
		if( (v1 & 0x40)==0x40 || (trans==0) ) PlotPixel(x+9, i, buffer, (v1 & 0x40)==0x40); 
		if( (v1 & 0x20)==0x20 || (trans==0) ) PlotPixel(x+10, i, buffer, (v1 & 0x20)==0x20);
		if( (v1 & 0x10)==0x10 || (trans==0) ) PlotPixel(x+11, i, buffer, (v1 & 0x10)==0x10);
		if( (v1 & 0x08)==0x08 || (trans==0) ) PlotPixel(x+12, i, buffer, (v1 & 0x08)==0x08);
		if( (v1 & 0x04)==0x04 || (trans==0) ) PlotPixel(x+13, i, buffer, (v1 & 0x04)==0x04);
		if( (v1 & 0x02)==0x02 || (trans==0) ) PlotPixel(x+14, i, buffer, (v1 & 0x02)==0x02);
		if( (v1 & 0x01)==0x01 || (trans==0) ) PlotPixel(x+15, i, buffer, (v1 & 0x01)==0x01);
		j++;
	}
}

void DDrawSprite8(char x, char y, unsigned char *buffer1, unsigned char *buffer2, char *bitmap, char *bitmap2, int trans)  
{
	int j, v1, v2, i;
	for (j=0;j<8;j++)
	{
		v1 = bitmap[j];
		v2 = bitmap2[j];
		if( (v1 & 0x80)==0x80 || (v2 & 0x08)==0x80 || (trans==0) ) {
			PlotPixel(x,   y+j, buffer1, (v1 & 0x80)==0x80);
			PlotPixel(x,   y+j, buffer2, (v2 & 0x80)==0x80);
		}
		if( (v1 & 0x40)==0x40 || (v2 & 0x40)==0x40 || (trans==0) ) {
			PlotPixel(x+1, y+j, buffer1, (v1 & 0x40)==0x40); 
			PlotPixel(x+1, y+j, buffer2, (v2 & 0x40)==0x40);
		}
		if( (v1 & 0x20)==0x20 || (v2 & 0x20)==0x20 || (trans==0) ) {
			PlotPixel(x+2, y+j, buffer1, (v1 & 0x20)==0x20);
			PlotPixel(x+2, y+j, buffer2, (v2 & 0x20)==0x20);
		}
		if( (v1 & 0x10)==0x10 || (v2 & 0x10)==0x10 || (trans==0) ) {
			PlotPixel(x+3, y+j, buffer1, (v1 & 0x10)==0x10);
			PlotPixel(x+3, y+j, buffer2, (v2 & 0x10)==0x10);
		}
		if( (v1 & 0x08)==0x08 || (v2 & 0x08)==0x08 || (trans==0) ) {
			PlotPixel(x+4, y+j, buffer1, (v1 & 0x08)==0x08);
			PlotPixel(x+4, y+j, buffer2, (v2 & 0x08)==0x08);
		}
		if( (v1 & 0x04)==0x04 || (v2 & 0x04)==0x04 || (trans==0) ) {
			PlotPixel(x+5, y+j, buffer1, (v1 & 0x04)==0x04);
			PlotPixel(x+5, y+j, buffer2, (v2 & 0x04)==0x04);
		}
		if( (v1 & 0x02)==0x02 || (v2 & 0x02)==0x02 || (trans==0) ) {
			PlotPixel(x+6, y+j, buffer1, (v1 & 0x02)==0x02);
			PlotPixel(x+6, y+j, buffer2, (v2 & 0x02)==0x02);
		}
		if( (v1 & 0x01)==0x01 || (v2 & 0x01)==0x01 || (trans==0) ) {
			PlotPixel(x+7, y+j, buffer1, (v1 & 0x01)==0x01);
			PlotPixel(x+7, y+j, buffer2, (v2 & 0x01)==0x01);
		}
	}
}

void DDrawSprite8x16(char x, char y, unsigned char *buffer1, unsigned char *buffer2, char *bitmap, char *bitmap2, int trans)
{
	int j, v1, v2;
	for (j=0;j<16;j++)
	{
		v1 = bitmap[j];
		v2 = bitmap2[j];
		if( (v1 & 0x80)==0x80 || (v2 & 0x08)==0x80 || (trans==0) ) {
			PlotPixel(x,   y+j, buffer1, (v1 & 0x80)==0x80);
			PlotPixel(x,   y+j, buffer2, (v2 & 0x80)==0x80);
		}
		if( (v1 & 0x40)==0x40 || (v2 & 0x40)==0x40 || (trans==0) ) {
			PlotPixel(x+1, y+j, buffer1, (v1 & 0x40)==0x40); 
			PlotPixel(x+1, y+j, buffer2, (v2 & 0x40)==0x40);
		}
		if( (v1 & 0x20)==0x20 || (v2 & 0x20)==0x20 || (trans==0) ) {
			PlotPixel(x+2, y+j, buffer1, (v1 & 0x20)==0x20);
			PlotPixel(x+2, y+j, buffer2, (v2 & 0x20)==0x20);
		}
		if( (v1 & 0x10)==0x10 || (v2 & 0x10)==0x10 || (trans==0) ) {
			PlotPixel(x+3, y+j, buffer1, (v1 & 0x10)==0x10);
			PlotPixel(x+3, y+j, buffer2, (v2 & 0x10)==0x10);
		}
		if( (v1 & 0x08)==0x08 || (v2 & 0x08)==0x08 || (trans==0) ) {
			PlotPixel(x+4, y+j, buffer1, (v1 & 0x08)==0x08);
			PlotPixel(x+4, y+j, buffer2, (v2 & 0x08)==0x08);
		}
		if( (v1 & 0x04)==0x04 || (v2 & 0x04)==0x04 || (trans==0) ) {
			PlotPixel(x+5, y+j, buffer1, (v1 & 0x04)==0x04);
			PlotPixel(x+5, y+j, buffer2, (v2 & 0x04)==0x04);
		}
		if( (v1 & 0x02)==0x02 || (v2 & 0x02)==0x02 || (trans==0) ) {
			PlotPixel(x+6, y+j, buffer1, (v1 & 0x02)==0x02);
			PlotPixel(x+6, y+j, buffer2, (v2 & 0x02)==0x02);
		}
		if( (v1 & 0x01)==0x01 || (v2 & 0x01)==0x01 || (trans==0) ) {
			PlotPixel(x+7, y+j, buffer1, (v1 & 0x01)==0x01);
			PlotPixel(x+7, y+j, buffer2, (v2 & 0x01)==0x01);
		}
	}
}

void DDrawSprite16(char x, char y, unsigned char *buffer1, unsigned char *buffer2, char *bitmap, char *bitmap2, int trans)
{
	int j, v1, v2, i;
	for (j=0;j<32;j++)
	{
		v1 = bitmap[j];
		v2 = bitmap2[j];

		i = y+(j/2);

		if( (v1 & 0x80)==0x80 || (v2 & 0x80)==0x80 || trans==0)
		{
			PlotPixel(x, i, buffer1, (v1 & 0x80)==0x80);
			PlotPixel(x, i, buffer2, (v2 & 0x80)==0x80);
		}
		if( (v1 & 0x40)==0x40 || (v2 & 0x40)==0x40 || trans==0)
		{
			PlotPixel(x+1, i, buffer1, (v1 & 0x40)==0x40);
			PlotPixel(x+1, i, buffer2, (v2 & 0x40)==0x40);
		}
		if( (v1 & 0x20)==0x20 || (v2 & 0x20)==0x20 || trans==0)
		{
			PlotPixel(x+2, i, buffer1, (v1 & 0x20)==0x20);
			PlotPixel(x+2, i, buffer2, (v2 & 0x20)==0x20);
		}
		if( (v1 & 0x10)==0x10 || (v2 & 0x10)==0x10 || trans==0)
		{
			PlotPixel(x+3, i, buffer1, (v1 & 0x10)==0x10);
			PlotPixel(x+3, i, buffer2, (v2 & 0x10)==0x10);
		}
		if( (v1 & 0x08)==0x08 || (v2 & 0x08)==0x08 || trans==0)
		{
			PlotPixel(x+4, i, buffer1, (v1 & 0x08)==0x08);
			PlotPixel(x+4, i, buffer2, (v2 & 0x08)==0x08);
		}
		if( (v1 & 0x04)==0x04 || (v2 & 0x04)==0x04 || trans==0)
		{
			PlotPixel(x+5, i, buffer1, (v1 & 0x04)==0x04);
			PlotPixel(x+5, i, buffer2, (v2 & 0x04)==0x04);
		}
		if( (v1 & 0x02)==0x02 || (v2 & 0x02)==0x02 || trans==0)
		{
			PlotPixel(x+6, i, buffer1, (v1 & 0x02)==0x02);
			PlotPixel(x+6, i, buffer2, (v2 & 0x02)==0x02);
		}
		if( (v1 & 0x01)==0x01 || (v2 & 0x01)==0x01 || trans==0)
		{
			PlotPixel(x+7, i, buffer1, (v1 & 0x01)==0x01);
			PlotPixel(x+7, i, buffer2, (v2 & 0x01)==0x01);
		}

		v1 = bitmap[j+1];
		v2 = bitmap2[j+1];

		if( (v1 & 0x80)==0x80 || (v2 & 0x80)==0x80 || trans==0)
		{
			PlotPixel(x+8,   i, buffer1, (v1 & 0x80)==0x80);
			PlotPixel(x+8,   i, buffer2, (v2 & 0x80)==0x80);
		}
		if( (v1 & 0x40)==0x40 || (v2 & 0x40)==0x40 || trans==0)
		{
			PlotPixel(x+9, i, buffer1, (v1 & 0x40)==0x40);
			PlotPixel(x+9, i, buffer2, (v2 & 0x40)==0x40);
		}
		if( (v1 & 0x20)==0x20 || (v2 & 0x20)==0x20 || trans==0)
		{
			PlotPixel(x+10, i, buffer1, (v1 & 0x20)==0x20);
			PlotPixel(x+10, i, buffer2, (v2 & 0x20)==0x20);
		}
		if( (v1 & 0x10)==0x10 || (v2 & 0x10)==0x10 || trans==0)
		{
			PlotPixel(x+11, i, buffer1, (v1 & 0x10)==0x10);
			PlotPixel(x+11, i, buffer2, (v2 & 0x10)==0x10);
		}
		if( (v1 & 0x08)==0x08 || (v2 & 0x08)==0x08 || trans==0)
		{
			PlotPixel(x+12, i, buffer1, (v1 & 0x08)==0x08);
			PlotPixel(x+12, i, buffer2, (v2 & 0x08)==0x08);
		}
		if( (v1 & 0x04)==0x04 || (v2 & 0x04)==0x04 || trans==0)
		{
			PlotPixel(x+13, i, buffer1, (v1 & 0x04)==0x04);
			PlotPixel(x+13, i, buffer2, (v2 & 0x04)==0x04);
		}
		if( (v1 & 0x02)==0x02 || (v2 & 0x02)==0x02 || trans==0)
		{
			PlotPixel(x+14, i, buffer1, (v1 & 0x02)==0x02);
			PlotPixel(x+14, i, buffer2, (v2 & 0x02)==0x02);
		}
		if( (v1 & 0x01)==0x01 || (v2 & 0x01)==0x01 || trans==0)
		{
			PlotPixel(x+15, i, buffer1, (v1 & 0x01)==0x01);
			PlotPixel(x+15, i, buffer2, (v2 & 0x01)==0x01);
		}
		j++;
	}
}

void DrawSprite32(char x, char y, unsigned char *buffer, char *bitmap, int trans)  
{
	int j, v1, i;
	for (j=0;j<128;j++)
	{
		v1 = bitmap[j];
		i = (y+j)/4;
		PlotPixel(x, i, buffer, (v1 & 0x80)==0x80);
		PlotPixel(x+1, i, buffer, (v1 & 0x40)==0x40); 
		PlotPixel(x+2, i, buffer, (v1 & 0x20)==0x20);
		PlotPixel(x+3, i, buffer, (v1 & 0x10)==0x10);
		PlotPixel(x+4, i, buffer, (v1 & 0x08)==0x08);
		PlotPixel(x+5, i, buffer, (v1 & 0x04)==0x04);
		PlotPixel(x+6, i, buffer, (v1 & 0x02)==0x02);
		PlotPixel(x+7, i, buffer, (v1 & 0x01)==0x01);
		v1 = bitmap[j+1];
		PlotPixel(x+8, i, buffer, (v1 & 0x80)==0x80);
		PlotPixel(x+9, i, buffer, (v1 & 0x40)==0x40); 
		PlotPixel(x+10, i, buffer, (v1 & 0x20)==0x20);
		PlotPixel(x+11, i, buffer, (v1 & 0x10)==0x10);
		PlotPixel(x+12, i, buffer, (v1 & 0x08)==0x08);
		PlotPixel(x+13, i, buffer, (v1 & 0x04)==0x04);
		PlotPixel(x+14, i, buffer, (v1 & 0x02)==0x02);
		PlotPixel(x+15, i, buffer, (v1 & 0x01)==0x01);
		v1 = bitmap[j+2];
		PlotPixel(x+16, i, buffer, (v1 & 0x80)==0x80);
		PlotPixel(x+17, i, buffer, (v1 & 0x40)==0x40); 
		PlotPixel(x+18, i, buffer, (v1 & 0x20)==0x20);
		PlotPixel(x+19, i, buffer, (v1 & 0x10)==0x10);
		PlotPixel(x+20, i, buffer, (v1 & 0x08)==0x08);
		PlotPixel(x+21, i, buffer, (v1 & 0x04)==0x04);
		PlotPixel(x+22, i, buffer, (v1 & 0x02)==0x02);
		PlotPixel(x+23, i, buffer, (v1 & 0x01)==0x01);
		v1 = bitmap[j+3];
		PlotPixel(x+24, i, buffer, (v1 & 0x80)==0x80);
		PlotPixel(x+25, i, buffer, (v1 & 0x40)==0x40); 
		PlotPixel(x+26, i, buffer, (v1 & 0x20)==0x20);
		PlotPixel(x+27, i, buffer, (v1 & 0x10)==0x10);
		PlotPixel(x+28, i, buffer, (v1 & 0x08)==0x08);
		PlotPixel(x+29, i, buffer, (v1 & 0x04)==0x04);
		PlotPixel(x+30, i, buffer, (v1 & 0x02)==0x02);
		PlotPixel(x+31, i, buffer, (v1 & 0x01)==0x01);
		j += 3;
	}
}
